# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

"""
This module defines the 'Main fields' page.

This module contains the following classes:
- LabelDelegate
- NatureDelegate
- EnthalpyDelegate
- CriterionDelegate
- CarrierDelegate
- StandardItemModelMainFields
- MainFieldsView
"""

#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------

import os, sys, string, types
import logging

#-------------------------------------------------------------------------------
# Third-party modules
#-------------------------------------------------------------------------------

from code_saturne.Base.QtCore    import *
from code_saturne.Base.QtGui     import *
from code_saturne.Base.QtWidgets import *

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

from code_saturne.Base.QtPage import ComboModel, RegExpValidator
from code_saturne.Base.QtPage import from_qvariant, to_text_string
from code_saturne.model.Common import LABEL_LENGTH_MAX, GuiParam

from code_saturne.Pages.MainFields import Ui_MainFields
from code_saturne.model.MainFieldsModel import MainFieldsModel

from code_saturne.model.LagrangianModel import LagrangianModel

#-------------------------------------------------------------------------------
# log config
#-------------------------------------------------------------------------------

logging.basicConfig()
log = logging.getLogger("MainFieldsView")
log.setLevel(GuiParam.DEBUG)

#-------------------------------------------------------------------------------
# EOS
#-------------------------------------------------------------------------------
EOS = 1
try:
   import eosAva
except:
   EOS = 0
else :
   import eosAva


#-------------------------------------------------------------------------------
# Line edit delegate for the label
#-------------------------------------------------------------------------------

class LabelDelegate(QItemDelegate):
    """
    Use of a QLineEdit in the table.
    """
    def __init__(self, parent=None):
        QItemDelegate.__init__(self, parent)
        self.parent = parent
        self.old_plabel = ""


    def createEditor(self, parent, option, index):
        editor = QLineEdit(parent)
        self.old_label = ""
        rx = "[_a-zA-Z][_A-Za-z0-9]{1," + str(LABEL_LENGTH_MAX-1) + "}"
        self.regExp = QRegExp(rx)
        v = RegExpValidator(editor, self.regExp)
        editor.setValidator(v)
        return editor


    def setEditorData(self, editor, index):
        editor.setAutoFillBackground(True)
        value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
        self.old_plabel = str(value)
        editor.setText(value)


    def setModelData(self, editor, model, index):
        if not editor.isModified():
            return

        if editor.validator().state == QValidator.Acceptable:
            new_plabel = str(editor.text())

            if new_plabel in model.mdl.getFieldLabelsList():
                default = {}
                default['label']  = self.old_plabel
                default['list']   = model.mdl.getFieldLabelsList()
                default['regexp'] = self.regExp
                log.debug("setModelData -> default = %s" % default)

                from code_saturne.Pages.VerifyExistenceLabelDialogView import VerifyExistenceLabelDialogView
                dialog = VerifyExistenceLabelDialogView(self.parent, default)
                if dialog.exec_():
                    result = dialog.get_result()
                    new_plabel = result['label']
                    log.debug("setModelData -> result = %s" % result)
                else:
                    new_plabel = self.old_plabel

            model.setData(index, new_plabel, Qt.DisplayRole)

#-------------------------------------------------------------------------------
# Combo box delegate for the nature of field
#-------------------------------------------------------------------------------

class NatureDelegate(QItemDelegate):
    """
    Use of a combo box in the table.
    """
    def __init__(self, parent):
        super(NatureDelegate, self).__init__(parent)
        self.parent   = parent


    def createEditor(self, parent, option, index):
        editor = QComboBox(parent)
        self.modelCombo = ComboModel(editor, 3, 1)
        self.modelCombo.addItem(self.tr("liquid"), 'liquid')
        self.modelCombo.addItem(self.tr("gas"), 'gas')
        self.modelCombo.addItem(self.tr("solid"), 'solid')

        row = index.row()
        if (row == 0) :
            self.modelCombo.disableItem(2)
        else :
            self.modelCombo.enableItem(2)

        editor.installEventFilter(self)
        return editor


    def setEditorData(self, comboBox, index):
        col = index.column()
        string = index.model().getData(index)[col]
        self.modelCombo.setItem(str_model=string)


    def setModelData(self, comboBox, model, index):
        txt = str(comboBox.currentText())
        value = self.modelCombo.dicoV2M[txt]
        log.debug("NatureDelegate value = %s"%value)

        selectionModel = self.parent.selectionModel()
        for idx in selectionModel.selectedIndexes():
            if idx.column() == index.column():
                model.setData(idx, value, Qt.DisplayRole)


#-------------------------------------------------------------------------------
# Combo box delegate for the enthalpy
#-------------------------------------------------------------------------------

class EnthalpyDelegate(QItemDelegate):
    """
    Use of a combo box in the table.
    """
    def __init__(self, parent, mdl):
        super(EnthalpyDelegate, self).__init__(parent)
        self.parent   = parent
        self.mdl      = mdl


    def createEditor(self, parent, option, index):
        editor = QComboBox(parent)
        predefined_flow = self.mdl.getPredefinedFlow()
        if predefined_flow in ["free_surface", "boiling_flow", "droplet_flow", "multiregime"]:
            self.modelCombo = ComboModel(editor, 2, 1)
            self.modelCombo.addItem(self.tr("off"), 'off')
            self.modelCombo.addItem(self.tr("total enthalpy"), 'total_enthalpy')
        else:
            self.modelCombo = ComboModel(editor, 3, 1)
            self.modelCombo.addItem(self.tr("off"), 'off')
            self.modelCombo.addItem(self.tr("total enthalpy"), 'total_enthalpy')
            self.modelCombo.addItem(self.tr("specific enthalpy"), 'specific_enthalpy')

        editor.installEventFilter(self)
        return editor


    def setEditorData(self, comboBox, index):
        col = index.column()
        string = index.model().getData(index)[col]
        self.modelCombo.setItem(str_model=string)


    def setModelData(self, comboBox, model, index):
        txt = str(comboBox.currentText())
        value = self.modelCombo.dicoV2M[txt]
        log.debug("EnthalpyDelegate value = %s"%value)

        selectionModel = self.parent.selectionModel()
        for idx in selectionModel.selectedIndexes():
            if idx.column() == index.column():
                model.setData(idx, value, Qt.DisplayRole)


#-------------------------------------------------------------------------------
# Combo box delegate for the criterion of field
#-------------------------------------------------------------------------------

class CriterionDelegate(QItemDelegate):
    """
    Use of a combo box in the table.
    """
    def __init__(self, parent):
        super(CriterionDelegate, self).__init__(parent)
        self.parent   = parent


    def createEditor(self, parent, option, index):
        editor = QComboBox(parent)
        self.modelCombo = ComboModel(editor, 3, 1)
        self.modelCombo.addItem(self.tr("continuous"), 'continuous')
        self.modelCombo.addItem(self.tr("dispersed"), 'dispersed')
        self.modelCombo.addItem(self.tr("auto"), 'auto')
        # TODO to delete if/when the auto option is implemented
        self.modelCombo.disableItem(2)
        # fixed to continuous for field 1
        if index.row() == 0 :
            editor.setEnabled(False)

        editor.installEventFilter(self)
        return editor


    def setEditorData(self, comboBox, index):
        row = index.row()
        col = index.column()
        string = index.model().getData(index)[col]
        self.modelCombo.setItem(str_model=string)


    def setModelData(self, comboBox, model, index):
        txt = str(comboBox.currentText())
        value = self.modelCombo.dicoV2M[txt]
        log.debug("CriterionDelegate value = %s"%value)

        selectionModel = self.parent.selectionModel()
        for idx in selectionModel.selectedIndexes():
            if idx.column() == index.column():
                model.setData(idx, value, Qt.DisplayRole)


#-------------------------------------------------------------------------------
# Combo box delegate for the carrier field
#-------------------------------------------------------------------------------


class CarrierDelegate(QItemDelegate):
    """
    Use of a combo box in the table.
    """
    def __init__(self, parent, mdl):
        super(CarrierDelegate, self).__init__(parent)
        self.parent   = parent
        self.mdl      = mdl


    def createEditor(self, parent, option, index):
        editor = QComboBox(parent)
        self.modelCombo = ComboModel(editor, 1, 1)
        fieldId = index.row()+1
        if self.mdl.getCriterion(fieldId) == "continuous" :
            self.modelCombo.addItem(self.tr("off"), 'off')
        else :
            for id in self.mdl.getContinuousFieldList() :
                label = self.mdl.getLabel(id)
                self.modelCombo.addItem(self.tr(label), label)

        editor.installEventFilter(self)
        return editor


    def setEditorData(self, comboBox, index):
        row = index.row()
        col = index.column()
        string = index.model().getData(index)[col]
        self.modelCombo.setItem(str_model=string)


    def setModelData(self, comboBox, model, index):
        txt = str(comboBox.currentText())
        value = self.modelCombo.dicoV2M[txt]
        log.debug("CarrierDelegate value = %s"%value)

        selectionModel = self.parent.selectionModel()
        for idx in selectionModel.selectedIndexes():
            if idx.column() == index.column():
                model.setData(idx, value, Qt.DisplayRole)


#-------------------------------------------------------------------------------
# StandardItemModelMainFields class
#-------------------------------------------------------------------------------

class StandardItemModelMainFields(QStandardItemModel):

    def __init__(self, mdl):
        """
        """
        QStandardItemModel.__init__(self)

        self.headers = [ self.tr("Field\nlabel"),
                         self.tr("Phase of\nfield"),
                         self.tr("Interfacial criterion"),
                         self.tr("Carrier phase"),
                         self.tr("Compressible"),
                         self.tr("Energy\nresolution")]

        self.setColumnCount(len(self.headers))

        self.tooltip = []

        self._data = []
        self.mdl = mdl


    def data(self, index, role):
        if not index.isValid():
            return None

        if role == Qt.ToolTipRole:
            return None

        elif role == Qt.DisplayRole:
            data = self._data[index.row()][index.column()]
            if index.column() in (0, 1, 2, 3, 5):
                if data:
                    return data
                else:
                    return None

        elif role == Qt.CheckStateRole:
            data = self._data[index.row()][index.column()]
            if index.column() == 4:
                if data == 'on':
                    return Qt.Checked
                else:
                    return Qt.Unchecked

        elif role == Qt.TextAlignmentRole:
            return Qt.AlignCenter

        return None


    def flags(self, index):

        if not index.isValid():
            return Qt.ItemIsEnabled
        if index.column() == 4:
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsUserCheckable
        elif index.column() == 5:
            if self.mdl.getPredefinedFlow() != "None" \
                    and self.mdl.getPredefinedFlow() != "particles_flow" \
                    and self.mdl.getHeatMassTransferStatus() == "on":
                return Qt.ItemIsSelectable
            else:
                return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable
        elif index.column() == 1 or index.column() == 2:

            if self.mdl.getPredefinedFlow() != "None" and (index.row()==0 or index.row()==1):
                return Qt.ItemIsEnabled | Qt.ItemIsSelectable
            elif self.mdl.getFieldNature(index.row()+1) == "solid" and index.column() == 2:
                return Qt.ItemIsEnabled | Qt.ItemIsSelectable
            else:
                return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable

        else:
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable


    def headerData(self, section, orientation, role):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return self.headers[section]
        return None


    def setData(self, index, value, role):
        if not index.isValid():
            return Qt.ItemIsEnabled

        # Update the row in the table
        row = index.row()
        col = index.column()
        FieldId = row + 1

        # Label
        if col == 0:
            new_plabel = from_qvariant(value, to_text_string)

            # Since all fields' labels are using the phase label, we rename
            # the labels in all formulas!
            old_plabel = self._data[row][col]
            for nf in self.mdl.case.xmlGetNodeList('formula'):
                if nf:
                    ftext = (str(nf).replace('<formula>','')).replace('</formula>','')
                    nf.xmlSetTextNode(ftext.replace(old_plabel, new_plabel))

            self._data[row][col] = new_plabel
            self.mdl.setLabel(FieldId, new_plabel)
            self.updateItem()

        # Nature of field
        elif col == 1:
            new_nature = from_qvariant(value, to_text_string)
            self._data[row][col] = new_nature
            self.mdl.setFieldNature(FieldId, new_nature)
            self.updateItem()

        # Interfacial criterion
        elif col == 2:
            new_crit = from_qvariant(value, to_text_string)
            self._data[row][col] = new_crit
            self.mdl.setCriterion(FieldId, new_crit)
            # update carrier field
            self.updateItem()

        # Carrier field
        elif col == 3:
            new_carrier = from_qvariant(value, to_text_string)
            self._data[row][col] = new_carrier
            # set carrier field Id in XML
            if self._data[row][col] != "off" :
               id = self.mdl.getFieldId(self._data[row][col])
            else :
               id = self._data[row][col]
            self.mdl.setCarrierField(FieldId, id)

        # Compressible
        elif col == 4:
            state = from_qvariant(value, int)
            if state == Qt.Unchecked:
                self._data[row][col] = "off"
                self.mdl.setCompressibleStatus(FieldId,"off")
            else:
                self._data[row][col] = "on"
                self.mdl.setCompressibleStatus(FieldId,"on")

        # Energy resolution
        elif col == 5:
            state = from_qvariant(value, to_text_string)
            self._data[row][col] = state
            self.mdl.setEnergyModel(FieldId, state)

        self.dataChanged.emit(index, index)
        return True


    def getData(self, index):
        row = index.row()
        return self._data[row]


    def newItem(self, existing_fieldId=None):
        """
        Add/load a field in the model.
        """
        row = self.rowCount()

        fieldId      = self.mdl.addField(existing_fieldId)
        label        = self.mdl.getLabel(fieldId)
        nature       = self.mdl.getFieldNature(fieldId)
        criterion    = self.mdl.getCriterion(fieldId)
        carrier      = self.mdl.getCarrierField(fieldId)
        carrierLabel = ""
        if carrier != "off" :
            carrierLabel = self.mdl.getLabel(carrier)
        else :
            carrierLabel = carrier
        compressible = self.mdl.getCompressibleStatus(fieldId)
        energy       = self.mdl.getEnergyModel(fieldId)

        field = [label, nature, criterion, carrierLabel, compressible, energy]

        self._data.append(field)
        self.setRowCount(row+1)


    def loadItem(self, label, nature, criterion, carrierLabel, compressible, energy):
        """
        Add/load a field in the model.
        """
        row = self.rowCount()

        field = [label, nature, criterion, carrierLabel, compressible, energy]

        self._data.append(field)
        self.setRowCount(row+1)


    def updateItem(self):
        # update carrier field and criterion
        for id in self.mdl.getFieldIdList() :
            carrier = self.mdl.getCarrierField(id)
            if carrier != "off" :
               self._data[int(id)-1][3] = self.mdl.getLabel(carrier)
            else :
               self._data[int(id)-1][3] = carrier
            self._data[int(id)-1][2] = self.mdl.getCriterion(id)


    def deleteItem(self, row):
        """
        Delete the row in the model.
        """
        del self._data[row]
        self.mdl.deleteField(row + 1)
        row = self.rowCount()
        self.setRowCount(row-1)
        self.updateItem()


#-------------------------------------------------------------------------------
# MainFieldsView class
#-------------------------------------------------------------------------------

class MainFieldsView(QWidget, Ui_MainFields):
    """
    Main fields layout.
    """
    def __init__(self, parent, case, tree):
        """
        Constructor
        """
        QWidget.__init__(self, parent)

        Ui_MainFields.__init__(self)
        self.setupUi(self)

        self.browser = tree
        self.case = case
        self.case.undoStopGlobal()
        self.mdl  = MainFieldsModel(self.case)
        self.lagr = LagrangianModel(self.case)

        # Main fields definition
        self.tableModelFields = StandardItemModelMainFields(self.mdl)
        self.tableViewFields.setModel(self.tableModelFields)
        self.tableViewFields.resizeColumnsToContents()
        self.tableViewFields.resizeRowsToContents()
        if QT_API == "PYQT4":
            self.tableViewFields.verticalHeader().setResizeMode(QHeaderView.ResizeToContents)
            self.tableViewFields.horizontalHeader().setResizeMode(QHeaderView.ResizeToContents)
            self.tableViewFields.horizontalHeader().setResizeMode(0,QHeaderView.Stretch)
        elif QT_API == "PYQT5":
            self.tableViewFields.verticalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
            self.tableViewFields.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
            self.tableViewFields.horizontalHeader().setSectionResizeMode(0,QHeaderView.Stretch)
        self.tableViewFields.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.tableViewFields.setSelectionMode(QAbstractItemView.SingleSelection)

        delegateLabel      = LabelDelegate(self.tableViewFields)
        delegateNature     = NatureDelegate(self.tableViewFields)
        delegateCriterion = CriterionDelegate(self.tableViewFields)
        delegateCarrier = CarrierDelegate(self.tableViewFields, self.mdl)
        delegateEnthalpy = EnthalpyDelegate(self.tableViewFields, self.mdl)

        self.tableViewFields.setItemDelegateForColumn(0, delegateLabel)
        self.tableViewFields.setItemDelegateForColumn(1, delegateNature)
        self.tableViewFields.setItemDelegateForColumn(2, delegateCriterion)
        self.tableViewFields.setItemDelegateForColumn(3, delegateCarrier)
        self.tableViewFields.setItemDelegateForColumn(5, delegateEnthalpy)

        model = self.mdl.getPredefinedFlow()
        if model != "None":
            self._hidePushButtons()
        else:
            self._initializePushButtons()

        for fieldId in self.mdl.getFieldIdList():
            label = self.mdl.getLabel(fieldId)
            nature = self.mdl.getFieldNature(fieldId)
            criterion = self.mdl.getCriterion(fieldId)
            carrier = self.mdl.getCarrierField(fieldId)
            if carrier != "off":
                carrierLabel = self.mdl.getLabel(carrier)
            else:
                carrierLabel = carrier
            compressible = self.mdl.getCompressibleStatus(fieldId)
            energy = self.mdl.getEnergyModel(fieldId)
            self.tableModelFields.loadItem(label, nature, criterion, carrierLabel, compressible, energy)
        self.browser.configureTree(self.case)

        self.case.undoStartGlobal()

    def _initializePushButtons(self):
        self.pushButtonAdd.clicked.connect(self.slotAddField)
        self.pushButtonDelete.clicked.connect(self.slotDeleteField)
        self.tableModelFields.dataChanged.connect(self.dataChanged)

        if len(self.mdl.getFieldIdList()) > 2:
            self.pushButtonDelete.setEnabled(1)
        else:
            self.pushButtonDelete.setEnabled(0)

    def _hidePushButtons(self):
        self.pushButtonAdd.hide()
        self.pushButtonDelete.hide()

    def dataChanged(self, topLeft, bottomRight):
        for row in range(topLeft.row(), bottomRight.row() + 1):
            self.tableViewFields.resizeRowToContents(row)
        for col in range(topLeft.column(), bottomRight.column() + 1):
            self.tableViewFields.resizeColumnToContents(col)

        self.browser.configureTree(self.case)


    @pyqtSlot()
    def slotAddField(self):
        """
        Add a Field.
        """
        self.tableViewFields.clearSelection()
        self.tableModelFields.newItem()

        if len(self.mdl.getFieldIdList()) > 2:
            self.pushButtonDelete.setEnabled(1)
        else:
            self.pushButtonDelete.setEnabled(0)

        self.browser.configureTree(self.case)


    @pyqtSlot()
    def slotDeleteField(self):
        """
        Delete the Field from the list (one by one).
        """
        row = self.tableViewFields.currentIndex().row()
        if row >= 0 :
            log.debug("slotDeleteProfile -> %s" % row)
            self.tableModelFields.deleteItem(row)

        if len(self.mdl.getFieldIdList()) > 2:
            self.pushButtonDelete.setEnabled(1)
        else:
            self.pushButtonDelete.setEnabled(0)

        self.browser.configureTree(self.case)

#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------



# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

"""
This module defines the 'Users control' page.

This module contains the following classes:
- NameDelegate
- LocationDelegate
- StandardItemModelUsersControl
- UsersControlView
"""

#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------

import os, sys, string, types
import logging

#-------------------------------------------------------------------------------
# Third-party modules
#-------------------------------------------------------------------------------

from code_saturne.Base.QtCore    import *
from code_saturne.Base.QtGui     import *
from code_saturne.Base.QtWidgets import *

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

from code_saturne.Base.QtPage import ComboModel, RegExpValidator
from code_saturne.Base.QtPage import from_qvariant, to_text_string
from code_saturne.Pages.UsersControl import Ui_UsersControl
from code_saturne.model.UsersControlModel import UsersControlModel
from code_saturne.model.Common import LABEL_LENGTH_MAX, GuiParam

#-------------------------------------------------------------------------------
# log config
#-------------------------------------------------------------------------------

logging.basicConfig()
log = logging.getLogger("UsersControl")
log.setLevel(GuiParam.DEBUG)

#-------------------------------------------------------------------------------
# lineEdit delegate for the name
#-------------------------------------------------------------------------------

class NameDelegate(QItemDelegate):
    """
    Use of a QLineEdit in the table.
    """
    def __init__(self, parent=None):
        QItemDelegate.__init__(self, parent)
        self.parent = parent
        self.old_pname = ""


    def createEditor(self, parent, option, index):
        editor = QLineEdit(parent)
        self.old_name = ""
        rx = "[_a-zA-Z][_A-Za-z0-9]{1," + str(LABEL_LENGTH_MAX-1) + "}"
        self.regExp = QRegExp(rx)
        v = RegExpValidator(editor, self.regExp)
        editor.setValidator(v)
        return editor


    def setEditorData(self, editor, index):
        editor.setAutoFillBackground(True)
        value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
        self.old_pname = str(value)
        editor.setText(value)


    def setModelData(self, editor, model, index):
        if not editor.isModified():
            return

        if editor.validator().state == QValidator.Acceptable:
            new_pname = str(editor.text())

            if new_pname in model.mdl.getUserNamesList():
                default = {}
                default['name']  = self.old_pname
                default['list']   = model.mdl.getUserNamesList()
                default['regexp'] = self.regExp
                log.debug("setModelData -> default = %s" % default)

                from code_saturne.Pages.VerifyExistenceLabelDialogView import VerifyExistenceLabelDialogView
                dialog = VerifyExistenceLabelDialogView(self.parent, default)
                if dialog.exec_():
                    result = dialog.get_result()
                    new_pname = result['name']
                    log.debug("setModelData -> result = %s" % result)
                else:
                    new_pname = self.old_pname

            model.setData(index, new_pname, Qt.DisplayRole)


#-------------------------------------------------------------------------------
# Location delegate
#-------------------------------------------------------------------------------

class LocationDelegate(QItemDelegate):
    """
    Use of a combo box in the table.
    """
    def __init__(self, parent):
        super(LocationDelegate, self).__init__(parent)
        self.parent = parent


    def createEditor(self, parent, option, index):
        editor = QComboBox(parent)
        self.modelCombo = ComboModel(editor, 4, 1)
        self.modelCombo.addItem(self.tr("cells"), "cells")
        self.modelCombo.addItem(self.tr("interior faces"), "internal")
        self.modelCombo.addItem(self.tr("boundary faces"), "boundary")
        self.modelCombo.addItem(self.tr("vertices"), "vertices")

        editor.installEventFilter(self)
        return editor


    def setEditorData(self, comboBox, index):
        row = index.row()
        col = index.column()
        string = index.model().getData(index)[col]
        self.modelCombo.setItem(str_model=string)


    def setModelData(self, comboBox, model, index):
        txt = str(comboBox.currentText())
        value = self.modelCombo.dicoV2M[txt]
        log.debug("LocationDelegate value = %s"%value)

        selectionModel = self.parent.selectionModel()
        for idx in selectionModel.selectedIndexes():
            if idx.column() == index.column():
                model.setData(idx, value, Qt.DisplayRole)


#-------------------------------------------------------------------------------
# Dimension delegate
#-------------------------------------------------------------------------------

class UserDimensionDelegate(QItemDelegate):
    """
    Use of a combo box in the table for the user array dimension.
    """

    def __init__(self, parent):
        super(UserDimensionDelegate, self).__init__(parent)
        self.parent = parent

    def createEditor(self, parent, option, index):
        editor = QComboBox(parent)
        self.modelCombo = ComboModel(editor, 5, 1)

        self.modelCombo.addItem(self.tr("1"), "1")
        self.modelCombo.addItem(self.tr("2"), "2")
        self.modelCombo.addItem(self.tr("3"), "3")
        self.modelCombo.addItem(self.tr("6"), "6")
        self.modelCombo.addItem(self.tr("9"), "9")

        editor.installEventFilter(self)
        return editor

    def setEditorData(self, comboBox, index):
        row = index.row()
        col = index.column()
        string = index.model().getData(index)[col]
        self.modelCombo.setItem(str_model=string)

    def setModelData(self, comboBox, model, index):
        txt = str(comboBox.currentText())
        value = self.modelCombo.dicoV2M[txt]
        log.debug("UserDimensionDelegate value = %s"%value)

        selectionModel = self.parent.selectionModel()
        for idx in selectionModel.selectedIndexes():
            if idx.column() == index.column():
                model.setData(idx, value, Qt.DisplayRole)


#-------------------------------------------------------------------------------
# StandardItemModelUsersControl class
#-------------------------------------------------------------------------------

class StandardItemModelUsersControl(QStandardItemModel):

    def __init__(self, parent, mdl):
        """
        """
        QStandardItemModel.__init__(self)

        self.headers = [self.tr("Name"),
                        self.tr("Location"),
                        self.tr("Dimension")]

        self.setColumnCount(len(self.headers))
        self.parent = parent

        self.tooltip = []

        self._data  = []
        self.mdl    = mdl


    def data(self, index, role):
        if not index.isValid():
            return None

        if role == Qt.ToolTipRole:
            return None

        elif role == Qt.DisplayRole:
            data = self._data[index.row()][index.column()]
            if data:
                return data
            else:
                return None

        return None


    def flags(self, index):
        if not index.isValid():
            return Qt.ItemIsEnabled
        return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable


    def headerData(self, section, orientation, role):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return self.headers[section]
        return None


    def setData(self, index, value, role):
        if not index.isValid():
            return Qt.ItemIsEnabled

        # Update the row in the table
        row = index.row()
        col = index.column()

        name = self._data[row][0]
        new_val = from_qvariant(value, to_text_string)

        # Name
        if col == 0:
            self.mdl.setUsersName(name, new_val)
            self._data[row][col] = new_val
        # location
        elif col == 1:
            self.mdl.setUsersLocation(name, new_val)
            self._data[row][col] = new_val
        # dimension
        elif col == 2:
            self.mdl.setUsersDim(name, new_val)
            self._data[row][col] = new_val

        return True


    def getData(self, index):
        row = index.row()
        return self._data[row]


    def newItem(self, name=None):
        """
        Add/load a scalar in the model.
        """
        row = self.rowCount()

        if name == None:
            name = self.mdl.addUser("User_" + str(row + 1))
        location = self.mdl.getUsersLocation(name)
        dim      = self.mdl.getUsersDim(name)

        scalar = [name, location, dim]

        self._data.append(scalar)
        self.setRowCount(row + 1)


    def deleteItem(self, row):
        """
        Delete the row in the model.
        """
        name = self._data[row][0]
        del self._data[row]
        self.mdl.deleteUser(name)
        row = self.rowCount()
        self.setRowCount(row - 1)


#-------------------------------------------------------------------------------
# UsersControlView class
#-------------------------------------------------------------------------------

class UsersControlView(QWidget, Ui_UsersControl):
    """
    UsersControlView layout.
    """
    def __init__(self, parent, case):
        """
        Constructor
        """
        QWidget.__init__(self, parent)

        Ui_UsersControl.__init__(self)
        self.setupUi(self)

        self.case = case
        self.case.undoStopGlobal()

        self.mdl = UsersControlModel(self.case)

        self.tableModelUsers = StandardItemModelUsersControl(self, self.mdl)
        self.tableViewUsers.setModel(self.tableModelUsers)
        if QT_API == "PYQT4":
            self.tableViewUsers.horizontalHeader().setResizeMode(0,QHeaderView.Stretch)
            self.tableViewUsers.horizontalHeader().setResizeMode(1,QHeaderView.Stretch)
            self.tableViewUsers.horizontalHeader().setResizeMode(2,QHeaderView.Stretch)
        elif QT_API == "PYQT5":
            self.tableViewUsers.horizontalHeader().setSectionResizeMode(0,QHeaderView.Stretch)
            self.tableViewUsers.horizontalHeader().setSectionResizeMode(1,QHeaderView.Stretch)
            self.tableViewUsers.horizontalHeader().setSectionResizeMode(2,QHeaderView.Stretch)
        self.tableViewUsers.resizeColumnsToContents()
        self.tableViewUsers.resizeRowsToContents()
        self.tableViewUsers.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.tableViewUsers.setSelectionMode(QAbstractItemView.SingleSelection)

        delegateName     = NameDelegate(self.tableViewUsers)
        delegateLocation = LocationDelegate(self.tableViewUsers)
        delegateDim      = UserDimensionDelegate(self.tableViewUsers)

        self.tableViewUsers.setItemDelegateForColumn(0, delegateName)
        self.tableViewUsers.setItemDelegateForColumn(1, delegateLocation)
        self.tableViewUsers.setItemDelegateForColumn(2, delegateDim)

        # Connections
        self.pushButtonAdd.clicked.connect(self.slotAddUsers)
        self.pushButtonDelete.clicked.connect(self.slotDeleteUsers)

        # load values
        for user in self.mdl.getUserNamesList():
            self.tableModelUsers.newItem(user)

        self.case.undoStartGlobal()


    @pyqtSlot()
    def slotAddUsers(self):
        """
        Add a scalar
        """
        self.tableViewUsers.clearSelection()
        self.tableModelUsers.newItem()


    @pyqtSlot()
    def slotDeleteUsers(self):
        """
        Delete the a scalar from the list (one by one).
        """
        row = self.tableViewUsers.currentIndex().row()
        if row >= 0 :
            log.debug("slotDeleteScalar -> %s" % row)
            self.tableModelUsers.deleteItem(row)



# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

"""
This module contains the following classes and function:
- NameDelegate
- ChemicalFormulaDelegate
- ValueDelegate
- StandardItemModelSpecies
- GasCombustionView
"""

#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------

import logging, os

#-------------------------------------------------------------------------------
# Third-party modules
#-------------------------------------------------------------------------------

from code_saturne.Base.QtCore    import *
from code_saturne.Base.QtGui     import *
from code_saturne.Base.QtWidgets import *

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

from code_saturne.model.Common import LABEL_LENGTH_MAX, GuiParam
from code_saturne.Base.QtPage import ComboModel, RegExpValidator
from code_saturne.Base.QtPage import DoubleValidator, from_qvariant
from code_saturne.Base.QtPage import to_text_string
from code_saturne.Base.QtPage import IntValidator
from code_saturne.Pages.GasCombustionForm import Ui_GasCombustionForm
from code_saturne.model.GasCombustionModel import GasCombustionModel
from code_saturne.model.GasCombustionModel import ThermochemistryData

#-------------------------------------------------------------------------------
# log config
#-------------------------------------------------------------------------------

logging.basicConfig()
log = logging.getLogger("GasCombustionView")
log.setLevel(GuiParam.DEBUG)

#-------------------------------------------------------------------------------
# Line edit delegate for the species label
#-------------------------------------------------------------------------------

class NameDelegate(QItemDelegate):
    """
    Use of a QLineEdit in the table.
    """
    def __init__(self, parent=None):
        QItemDelegate.__init__(self, parent)
        self.parent = parent
        self.old_pname = ""


    def createEditor(self, parent, option, index):
        editor = QLineEdit(parent)
        self.old_pname = ""
        rx = "[_a-zA-Z][_A-Za-z0-9]{1," + str(LABEL_LENGTH_MAX-1) + "}"
        self.regExp = QRegExp(rx)
        v = RegExpValidator(editor, self.regExp)
        editor.setValidator(v)
        return editor


    def setEditorData(self, editor, index):
        editor.setAutoFillBackground(True)
        value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
        self.old_pname = str(value)
        editor.setText(value)


    def setModelData(self, editor, model, index):
        if not editor.isModified():
            return

        if editor.validator().state == QValidator.Acceptable:
            new_pname = str(editor.text())

            if new_pname in model.mdl.getSpeciesNamesList():
                default = {}
                default['name']  = self.old_pname
                default['list']   = model.mdl.getSpeciesNamesList()
                default['regexp'] = self.regExp
                log.debug("setModelData -> default = %s" % default)

                from code_saturne.Pages.VerifyExistenceLabelDialogView import VerifyExistenceLabelDialogView
                dialog = VerifyExistenceLabelDialogView(self.parent, default)
                if dialog.exec_():
                    result = dialog.get_result()
                    new_pname = result['name']
                    log.debug("setModelData -> result = %s" % result)
                else:
                    new_pname = self.old_pname

            model.setData(index, new_pname, Qt.DisplayRole)

#-------------------------------------------------------------------------------
# Line edit delegate for the chemical formula
#-------------------------------------------------------------------------------

class ChemicalFormulaDelegate(QItemDelegate):
    """
    Use of a QLineEdit in the table.
    """
    def __init__(self, parent=None):
        QItemDelegate.__init__(self, parent)
        self.parent = parent
        self.old_pname = ""


    def createEditor(self, parent, option, index):
        editor = QLineEdit(parent)
        self.old_pname = ""
        rx = "[chonslCHONSL()][CHONSLchonsl()0-9]{0," + str(LABEL_LENGTH_MAX-1) + "}"
        self.regExp = QRegExp(rx)
        v = RegExpValidator(editor, self.regExp)
        editor.setValidator(v)
        return editor


    def setEditorData(self, editor, index):
        editor.setAutoFillBackground(True)
        value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
        self.old_pname = str(value)
        editor.setText(value)


    def setModelData(self, editor, model, index):
        if not editor.isModified():
            return

        if editor.validator().state == QValidator.Acceptable:
            new_pname = str(editor.text())

            if new_pname in model.mdl.getSpeciesNamesList():
                default = {}
                default['name']  = self.old_pname
                default['list']   = model.mdl.getSpeciesNamesList()
                default['regexp'] = self.regExp
                log.debug("setModelData -> default = %s" % default)

                from code_saturne.Pages.VerifyExistenceLabelDialogView import VerifyExistenceLabelDialogView
                dialog = VerifyExistenceLabelDialogView(self.parent, default)
                if dialog.exec_():
                    result = dialog.get_result()
                    new_pname = result['name']
                    log.debug("setModelData -> result = %s" % result)
                else:
                    new_pname = self.old_pname

            model.setData(index, new_pname, Qt.DisplayRole)

#-------------------------------------------------------------------------------
# Line edit delegate for the value
#-------------------------------------------------------------------------------

class ValueDelegate(QItemDelegate):
    def __init__(self, parent=None):
        super(ValueDelegate, self).__init__(parent)
        self.parent = parent


    def createEditor(self, parent, option, index):
        editor = QLineEdit(parent)
        v = DoubleValidator(editor, min=0.)
        editor.setValidator(v)
        return editor


    def setEditorData(self, editor, index):
        editor.setAutoFillBackground(True)
        value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
        editor.setText(value)


    def setModelData(self, editor, model, index):
        if not editor.isModified():
            return
        if editor.validator().state == QValidator.Acceptable:
            value = from_qvariant(editor.text(), float)
            for idx in self.parent.selectionModel().selectedIndexes():
                if idx.column() == index.column():
                    model.setData(idx, value, Qt.DisplayRole)


#-------------------------------------------------------------------------------
# StandarItemModel class
#-------------------------------------------------------------------------------

class StandardItemModelSpecies(QStandardItemModel):
    """
    """
    def __init__(self, parent, mdl):
        """
        """
        QStandardItemModel.__init__(self)

        self.headers = [self.tr("Species"),
                        self.tr("Chemical Formula"),
                        self.tr("Nb of moles (Fuel)"),
                        self.tr("Nb of moles (Oxidiser)"),
                        self.tr("Nb of moles (Product)"),
                        self.tr("Absorption Coeff")]

        self.setColumnCount(len(self.headers))

        self._data = []
        self.parent = parent
        self.mdl  = mdl


    def data(self, index, role):
        if not index.isValid():
            return None

        row = index.row()
        col = index.column()

        if role == Qt.DisplayRole:
            return self._data[row][col]

        return None


    def flags(self, index):
        if not index.isValid():
            return Qt.ItemIsEnabled
        if index.column() != 0:
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable
        else:
            return Qt.ItemIsSelectable


    def headerData(self, section, orientation, role):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return self.headers[section]
        return None


    def setData(self, index, value, role):
        if not index.isValid():
            return Qt.ItemIsEnabled

        # Update the row in the table
        row = index.row()
        col = index.column()

        # Label (nothing to do for the species label)
        if col == 0:
            pass

        # Chemical formula
        elif col == 1:
            ChemicalFormula = str(from_qvariant(value, to_text_string))
            self._data[row][col] = ChemicalFormula
            label = self._data[row][0]
            self.mdl.setSpeciesChemicalFormula(label, ChemicalFormula)

        # Fuel Composition
        elif col == 2:
            CompFuel = str(from_qvariant(value, to_text_string))
            self._data[row][col] = CompFuel
            label = self._data[row][0]
            self.mdl.setCompFuel(label, CompFuel)

        # Oxi Composition
        elif col == 3:
            CompOxi = str(from_qvariant(value, to_text_string))
            self._data[row][col] = CompOxi
            label = self._data[row][0]
            self.mdl.setCompOxi(label, CompOxi)

        # Product Composition
        elif col == 4:
            CompProd = str(from_qvariant(value, to_text_string))
            self._data[row][col] = CompProd
            label = self._data[row][0]
            self.mdl.setCompProd(label, CompProd)

        # Coeff absorption
        elif col == 5:
            CoeffAbsorp = str(from_qvariant(value, to_text_string))
            self._data[row][col] = CoeffAbsorp
            label = self._data[row][0]
            self.mdl.setCoeffAbsorp(label, CoeffAbsorp)

        self.dataChanged.emit(index, index)
        return True


    def getData(self, index):
        row = index.row()
        return self._data[row]


    def newItem(self, existing_name=None):
        """
        Add an item in the table view
        """
        row = self.rowCount()

        label = self.mdl.addSpecies(existing_name)

        ChemicalFormula = self.mdl.getSpeciesChemicalFormula(label)
        CompFuel = self.mdl.getCompFuel(label)
        CompOxi = self.mdl.getCompOxi(label)
        CompProd = self.mdl.getCompProd(label)
        CoeffAbsorp = self.mdl.getCoeffAbsorp(label)

        species = [label, ChemicalFormula, CompFuel, CompOxi, CompProd, CoeffAbsorp]

        self.setRowCount(row+1)
        self._data.append(species)


    def getItem(self, row):
        """
        Return the values for an item.
        """
        [label, ChemicalFormula, CompFuel, CompOxi, CompProd, CoeffAbsorp] = self._data[row]
        return label, ChemicalFormula, CompFuel, CompOxi, CompProd, CoeffAbsorp


    def deleteItem(self, row):
        """
        Delete the row in the model.
        """
        log.debug("deleteItem row = %i " % row)

        del self._data[row]
        row = self.rowCount()
        self.setRowCount(row-1)


#-------------------------------------------------------------------------------
# Main class
#-------------------------------------------------------------------------------

class GasCombustionView(QWidget, Ui_GasCombustionForm):
    """
    Class to open the Gas Combustion option Page.
    """

    def __init__(self, parent, case):
        """
        Constructor
        """
        QWidget.__init__(self, parent)

        Ui_GasCombustionForm.__init__(self)
        self.setupUi(self)

        self.case = case
        self.case.undoStopGlobal()
        self.mdl = GasCombustionModel(self.case)
        self.thermodata = ThermochemistryData(self.case)

        # Model for table View
        self.modelSpecies = StandardItemModelSpecies(self, self.thermodata)
        self.tableViewSpecies.setModel(self.modelSpecies)

        # Delegates
        delegateLabel            = NameDelegate(self.tableViewSpecies)
        delegateChemicalFormula  = ChemicalFormulaDelegate(self.tableViewSpecies)
        delegateCompFuel  = ValueDelegate(self.tableViewSpecies)
        delegateCompOxi   = ValueDelegate(self.tableViewSpecies)
        delegateCompProd  = ValueDelegate(self.tableViewSpecies)
        delegateCoeffAbsorp  = ValueDelegate(self.tableViewSpecies)

        self.tableViewSpecies.setItemDelegateForColumn(0, delegateLabel)
        self.tableViewSpecies.setItemDelegateForColumn(1, delegateChemicalFormula)
        self.tableViewSpecies.setItemDelegateForColumn(2, delegateCompFuel)
        self.tableViewSpecies.setItemDelegateForColumn(3, delegateCompOxi)
        self.tableViewSpecies.setItemDelegateForColumn(4, delegateCompProd)
        self.tableViewSpecies.setItemDelegateForColumn(5, delegateCoeffAbsorp)

        # tableView
        if QT_API == "PYQT4":
            self.tableViewSpecies.horizontalHeader().setResizeMode(QHeaderView.Stretch)
            self.tableViewSpecies.horizontalHeader().setResizeMode(0,QHeaderView.ResizeToContents)
            self.tableViewSpecies.horizontalHeader().setResizeMode(1,QHeaderView.ResizeToContents)
        elif QT_API == "PYQT5":
            self.tableViewSpecies.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
            self.tableViewSpecies.horizontalHeader().setSectionResizeMode(0,QHeaderView.ResizeToContents)
            self.tableViewSpecies.horizontalHeader().setSectionResizeMode(1,QHeaderView.ResizeToContents)

        # Set models and number of elements for combo boxes
        self.modelGasCombustionOption = ComboModel(self.comboBoxGasCombustionOption,1,1)

        # Combo models to choose the mode to create the Janaf File

        self.userModeForChemicalReaction   = ComboModel(self.comboBoxUserChoice, 2, 1)

        self.userModeForChemicalReaction.addItem("Automatic definition", 'auto')
        self.userModeForChemicalReaction.addItem("Defined by user", 'user')

        # Connections
        self.comboBoxUserChoice.activated[str].connect(self.slotUserChoice)
        self.comboBoxGasCombustionOption.activated[str].connect(self.slotGasCombustionOption)
        self.pushButtonThermochemistryData.pressed.connect(self.__slotSearchThermochemistryData)
        self.radioButtonCreateJanafFile.clicked.connect(self.slotCreateJanafFile)
        self.lineEditNbPointsTabu.textChanged[str].connect(self.slotNbPointsTabu)
        self.lineEditMaximumTemp.textChanged[str].connect(self.slotMaximumTemp)
        self.lineEditMinimumTemp.textChanged[str].connect(self.slotMinimumTemp)
        self.pushButtonAddSpecies.clicked.connect(self.slotAddSpecies)
        self.pushButtonDeleteSpecies.clicked.connect(self.slotDeleteSpecies)
        self.pushButtonGenerateJanafFile.clicked.connect(self.slotGenerateJanafFile)
        self.lineEditFuel.textChanged[str].connect(self.slotFuel)
        self.lineEditO2.textChanged[str].connect(self.slotVolPropO2)
        self.lineEditN2.textChanged[str].connect(self.slotVolPropN2)
        self.lineEditCOyield.textChanged[str].connect(self.slotCOyield)
        self.lineEditCSyield.textChanged[str].connect(self.slotCSyield)
        self.modelSpecies.dataChanged.connect(self.dataChanged)

        # Validators
        validatorNbPointsTabu = IntValidator(self.lineEditNbPointsTabu, min=1)
        validatorMaximumTemp  = DoubleValidator(self.lineEditMaximumTemp, min=273.0)
        validatorMinimumTemp  = DoubleValidator(self.lineEditMinimumTemp, min=273.0)
        rx = "[chonlCHONL()][CHONLchonl()0-9]{0," + str(LABEL_LENGTH_MAX-1) + "}"
        validatorFuel         = RegExpValidator(self.lineEditFuel,QRegExp(rx))
        validatorO2  = DoubleValidator(self.lineEditO2, min=1e-12, max=1.0)
        validatorN2  = DoubleValidator(self.lineEditN2, min=0.0, max=1.0)
        validatorCOyield  = DoubleValidator(self.lineEditCOyield, min=0.0)
        validatorCSyield  = DoubleValidator(self.lineEditCSyield, min=0.0)

        self.lineEditNbPointsTabu.setValidator(validatorNbPointsTabu)
        self.lineEditMaximumTemp.setValidator(validatorMaximumTemp)
        self.lineEditMinimumTemp.setValidator(validatorMinimumTemp)
        self.lineEditFuel.setValidator(validatorFuel)
        self.lineEditO2.setValidator(validatorO2)
        self.lineEditN2.setValidator(validatorN2)
        self.lineEditCOyield.setValidator(validatorCOyield)
        self.lineEditCSyield.setValidator(validatorCSyield)

        NbPointsTabu = self.thermodata.getNbPointsTabu()
        MaximumTemp  = self.thermodata.getMaximumTemp()
        MinimumTemp  = self.thermodata.getMinimumTemp()
        Option_UserMode = self.thermodata.getUserModeForChemicalReaction()
        ChemicalFormulaFuel = self.thermodata.getChemicalFormulaFuel()
        VolPropO2 = self.thermodata.getVolPropO2()
        VolPropN2 = self.thermodata.getVolPropN2()
        COyield = self.thermodata.getCOyield()
        CSyield = self.thermodata.getCSyield()

        self.lineEditNbPointsTabu.setText(str(NbPointsTabu))
        self.lineEditMaximumTemp.setText(str(MaximumTemp))
        self.lineEditMinimumTemp.setText(str(MinimumTemp))
        self.lineEditFuel.setText(str(ChemicalFormulaFuel))
        self.lineEditO2.setText(str(VolPropO2))
        self.lineEditN2.setText(str(VolPropN2))
        self.lineEditCOyield.setText(str(COyield))
        self.lineEditCSyield.setText(str(CSyield))

        # Initialize Widgets

        self.tableViewSpecies.reset()
        self.modelSpecies = StandardItemModelSpecies(self, self.thermodata)
        self.tableViewSpecies.setModel(self.modelSpecies)

        model = self.mdl.getGasCombustionModel()

        if model == 'd3p':
            self.modelGasCombustionOption.addItem(self.tr("adiabatic model"), "adiabatic")
            self.modelGasCombustionOption.addItem(self.tr("non adiabatic model"), "extended")
        elif model == 'ebu':
            self.modelGasCombustionOption.addItem(self.tr("reference Spalding model"), "spalding")
            self.modelGasCombustionOption.addItem(self.tr("extended model with enthalpy source term"), "enthalpy_st")
            self.modelGasCombustionOption.addItem(self.tr("extended model with mixture fraction transport"), "mixture_st")
            self.modelGasCombustionOption.addItem(self.tr("extended model with enthalpy and mixture fraction transport"), "enthalpy_mixture_st")
        elif model == 'lwp':
            self.modelGasCombustionOption.addItem(self.tr("reference two-peak model with adiabatic condition"), "2-peak_adiabatic")
            self.modelGasCombustionOption.addItem(self.tr("reference two-peak model with enthalpy source term"), "2-peak_enthalpy")
            self.modelGasCombustionOption.addItem(self.tr("reference three-peak model with adiabatic condition"), "3-peak_adiabatic")
            self.modelGasCombustionOption.addItem(self.tr("reference three-peak model with enthalpy source term"), "3-peak_enthalpy")
            self.modelGasCombustionOption.addItem(self.tr("reference four-peak model with adiabatic condition"), "4-peak_adiabatic")
            self.modelGasCombustionOption.addItem(self.tr("reference four-peak model with enthalpy source term"), "4-peak_enthalpy")

        option = self.mdl.getGasCombustionOption()
        self.modelGasCombustionOption.setItem(str_model= option)

        name = self.mdl.getThermoChemistryDataFileName()
        if name != None and name != "":
            self.labelThermochemistryFile.setText(str(name))
            self.pushButtonThermochemistryData.setStyleSheet("background-color: green")
        else:
            self.pushButtonThermochemistryData.setStyleSheet("background-color: red")

        self.radioButtonCreateJanafFile.hide()

        if self.thermodata.getCreateThermoDataFile() == 'on':
            self.radioButtonCreateJanafFile.setChecked(True)
            self.userModeForChemicalReaction.setItem(str_model= Option_UserMode)
            for label in self.thermodata.getSpeciesNamesList():
                self.modelSpecies.newItem(label)
        else:
            self.radioButtonCreateJanafFile.setChecked(False)

        # for the moment the option to create Janaf file in the GUI is only available with d3p
        if model == 'd3p':
            self.radioButtonCreateJanafFile.show()
            self.groupBoxTabulation.show()
            self.groupBoxChemicalReaction.show()
            self.groupBoxDefinedByUser.show()
            self.groupBoxGenerateDataFile.show()
            self.groupBoxAutomatic.show()

        self.slotCreateJanafFile()

        self.case.undoStartGlobal()


    @pyqtSlot(str)
    def slotGasCombustionOption(self, text):
        """
        Private slot.
        Binding method for gas combustion models.
        """
        option = self.modelGasCombustionOption.dicoV2M[str(text)]
        self.mdl.setGasCombustionOption(option)


    @pyqtSlot()
    def __slotSearchThermochemistryData(self):
        """
        Select a properties file of data for electric arc
        """
        data = self.case['data_path']
        if not data:
            data = "."
        title = self.tr("Thermochemistry file of data.")
        filetypes = self.tr("Thermochemistry (*dp_*);;All Files (*)")
        file = QFileDialog.getOpenFileName(self, title, data, filetypes)[0]
        file = str(file)
        if not file:
            return
        file = os.path.basename(file)
        if file not in os.listdir(data):
            title = self.tr("WARNING")
            msg   = self.tr("This selected file is not in the DATA directory")
            QMessageBox.information(self, title, msg)
        else:
            self.labelThermochemistryFile.setText(str(file))
            self.mdl.setThermoChemistryDataFileName(file)
            self.pushButtonThermochemistryData.setStyleSheet("background-color: green")

    @pyqtSlot()
    def slotCreateJanafFile(self):
        """
        Determine if the Thermochemistry file is created with the GUI.
        """
        if self.radioButtonCreateJanafFile.isChecked():
            self.thermodata.setCreateThermoDataFile("on")
            self.groupBoxTabulation.show()
            self.groupBoxChemicalReaction.show()
            self.lineEditNbPointsTabu.setText(str(self.thermodata.getNbPointsTabu()))
            self.lineEditMaximumTemp.setText(str(self.thermodata.getMaximumTemp()))
            self.lineEditMinimumTemp.setText(str(self.thermodata.getMinimumTemp()))
            self.slotUserChoice()
            return
        else:
            self.thermodata.setCreateThermoDataFile("off")
            self.groupBoxTabulation.hide()
            self.groupBoxChemicalReaction.hide()
            self.groupBoxDefinedByUser.hide()
            self.groupBoxGenerateDataFile.hide()
            self.groupBoxAutomatic.hide()

    @pyqtSlot(str)
    def slotUserChoice(self):
        """
        """
        model = self.userModeForChemicalReaction.dicoV2M[str(self.comboBoxUserChoice.currentText())]
        self.thermodata.setUserModeForChemicalReaction(model)
        self.groupBoxGenerateDataFile.show()
        if model == 'auto':
            self.groupBoxAutomatic.show()
            self.groupBoxDefinedByUser.hide()
            #update of the xml from the GUI
            self.slotFuel(self.lineEditFuel.text())
            self.slotVolPropO2(self.lineEditO2.text())
            self.slotVolPropN2(self.lineEditN2.text())
            self.slotCOyield(self.lineEditCOyield.text())
            self.slotCSyield(self.lineEditCSyield.text())
        elif model == 'user':
            self.groupBoxDefinedByUser.show()
            self.groupBoxAutomatic.hide()
            #update of the xml from the tableView
            row_tab = self.modelSpecies.rowCount()
            for row in range(row_tab):
                data = self.modelSpecies.getItem(row)
                self.thermodata.updateSpecies(data)

    @pyqtSlot(str)
    def slotNbPointsTabu(self, text):
        """
        Input Number of points for the tabulation (ENTH-TEMP)
        """
        if self.lineEditNbPointsTabu.validator().state == QValidator.Acceptable:
            NbPointsTabu = from_qvariant(text, int)
            self.thermodata.setNbPointsTabu(NbPointsTabu)

    @pyqtSlot(str)
    def slotMaximumTemp(self, text):
        """
        Input Maximum temperature for the tabulation (ENTH-TEMP)
        """
        if self.lineEditMaximumTemp.validator().state == QValidator.Acceptable:
            MaximumTemp = from_qvariant(text, float)
            self.thermodata.setMaximumTemp(MaximumTemp)

    @pyqtSlot(str)
    def slotMinimumTemp(self, text):
        """
        Input Minimum temperature for the tabulation (ENTH-TEMP)
        """
        if self.lineEditMinimumTemp.validator().state == QValidator.Acceptable:
            MinimumTemp = from_qvariant(text, float)
            self.thermodata.setMinimumTemp(MinimumTemp)

    @pyqtSlot(str)
    def slotFuel(self, text):
        """
        Input the chemical formula for the Fuel
        """
        if self.lineEditFuel.validator().state == QValidator.Acceptable:
            ChemicalFormula = from_qvariant(text, to_text_string)
            self.thermodata.setChemicalFormulaFuel(ChemicalFormula)

    @pyqtSlot(str)
    def slotVolPropO2(self, text):
        """
        Input volume proportion for O2
        """
        if self.lineEditO2.validator().state == QValidator.Acceptable:
            VolPropO2 = from_qvariant(text, float)
            self.thermodata.setVolPropO2(VolPropO2)
            self.lineEditN2.setText(str(round(1.0 - VolPropO2,12)))

    @pyqtSlot(str)
    def slotVolPropN2(self, text):
        """
        Input volume proportion for N2
        """
        if self.lineEditN2.validator().state == QValidator.Acceptable:
            VolPropN2 = from_qvariant(text, float)
            self.thermodata.setVolPropN2(VolPropN2)
            self.lineEditO2.setText(str(round(1.0 - VolPropN2,12)))

    @pyqtSlot(str)
    def slotCOyield(self, text):
        """
        Input the CO yield
        """
        if self.lineEditCOyield.validator().state == QValidator.Acceptable:
            COyield = from_qvariant(text, float)
            self.thermodata.setCOyield(COyield)

    @pyqtSlot(str)
    def slotCSyield(self, text):
        """
        Input the CS yield
        """
        if self.lineEditCSyield.validator().state == QValidator.Acceptable:
            CSyield = from_qvariant(text, float)
            self.thermodata.setCSyield(CSyield)

    @pyqtSlot()
    def slotAddSpecies(self):
        """
        Add a new item in the table when the 'Create' button is pushed.
        """
        self.tableViewSpecies.clearSelection()
        self.modelSpecies.newItem()

    @pyqtSlot()
    def slotDeleteSpecies(self):
        """
        Just delete the current selected entries from the table and
        of course from the XML file.
        """
        lst = []
        for index in self.tableViewSpecies.selectionModel().selectedRows():
            row = index.row()
            lst.append(row)

        lst.sort()
        lst.reverse()

        for row in lst:
            label = self.modelSpecies.getItem(row)[0]
            self.thermodata.deleteSpecies(label)
            self.modelSpecies.deleteItem(row)

        self.tableViewSpecies.clearSelection()

    @pyqtSlot()
    def slotGenerateJanafFile(self):
        """
        Generate the Thermochemistry file.
        """

        data = self.case['data_path']
        if not data:
            data = "."
        filename = "dp_ThermochemistryFromGui"
        file_path = os.path.join(data, filename)

        self.thermodata.WriteThermochemistryDataFile(file_path)

        self.mdl.setThermoChemistryDataFileName(filename)

        if self.thermodata.Error_GUI :
            self.labelThermochemistryFile.setText(str("Error in : "+filename))
            self.pushButtonGenerateJanafFile.setStyleSheet("background-color: red")
            self.pushButtonThermochemistryData.setStyleSheet("background-color: orange")
        else :
            self.labelThermochemistryFile.setText(str(filename))
            self.pushButtonGenerateJanafFile.setStyleSheet("")
            self.pushButtonThermochemistryData.setStyleSheet("background-color: green")


    @pyqtSlot("QModelIndex, QModelIndex")
    def dataChanged(self, topLeft, bottomRight):
        for row in range(topLeft.row(), bottomRight.row()+1):
            self.tableViewSpecies.resizeRowToContents(row)
        for col in range(topLeft.column(), bottomRight.column()+1):
            self.tableViewSpecies.resizeColumnToContents(col)


#-------------------------------------------------------------------------------
# Testing part
#-------------------------------------------------------------------------------


if __name__ == "__main__":
    pass


#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

import sys, os

from collections import OrderedDict
try:
    import ConfigParser  # Python2
    configparser = ConfigParser
except Exception:
    import configparser  # Python3

from code_saturne import cs_batch

#-------------------------------------------------------------------------------
# Get info on locally installed version
#-------------------------------------------------------------------------------

def get_install_config_info(pkg):
    """
    Return info based on install configuration:
    dictionnary with 'batch', 'submit_command', 'resource', 'compute_builds'
    """

    c = OrderedDict({'batch': None,
                     'submit_command': None,
                     'resource_name': None,
                     'compute_builds': None})

    # Use alternate compute (back-end) package if defined

    config = configparser.ConfigParser()
    config.read(pkg.get_configfiles())

    for o in ('batch', 'submit_command', 'resource_name'):
        if config.has_option('install', o):
            c[o] = config.get('install', o)

    if config.has_option('install', 'compute_versions'):
        c['compute_builds'] = config.get('install',
                                         'compute_versions').split(':')

    if c['resource_name']:
        c['resource_name'] = c['resource_name'].lower()

    return c

#-------------------------------------------------------------------------------
# Get local resource name based on installation info
#-------------------------------------------------------------------------------

def get_resource_name(i_c):
    """
    Return name of current resource, based on dictionnary returned
    from get_install_config_info.
    """

    # Resources: try to find a matching section, using
    # resource_name, batch, and job_defaults in decreasing priority.

    resource_name = i_c['resource_name']
    if not resource_name:
        resource_name = i_c['batch']
        if resource_name:
            resource_name = os.path.basename(resource_name).lower()
    if not resource_name:
        resource_name = 'job_defaults'

    return resource_name

#===============================================================================
# Class used to manage run configuration files
#===============================================================================

class run_conf(object):

    def __init__(self,
                 path,
                 package=None,
                 create_if_missing=True,
                 rebuild=False):
        """
        Initialize run configuration info object.
        """

        self.path = path
        self.package = package
        self.sections = OrderedDict()
        self.comments = {}

        if not path:
            return

        need_rebuild = rebuild
        force_rebuild = False

        lines = []

        try:
            f = open(self.path, mode = 'r')
            lines = f.readlines()
            f.close()

        except IOError:
            if create_if_missing or rebuild:
                need_rebuild = True
                force_rebuild = True
                lines = []
            else:
                print("Error: can not open or read %s\n" % self.path)
                sys.exit(1)

        for i in range(len(lines)):
            lines[i] = lines[i].rstrip()

        self.__parse__(lines)

        if need_rebuild:
            self.rebuild_resource(force_rebuild)

    #---------------------------------------------------------------------------

    def rebuild_resource(self, force_rebuild=False):
        """
        Rebuild dictionnary using reference templates
        """

        i_c = get_install_config_info(self.package)
        resource_name = get_resource_name(i_c)

        if not resource_name in self.sections:
            self.sections[resource_name] = OrderedDict()

        # Determine or build default job name for batch

        topdir, scriptdir = os.path.split(os.path.split(self.path)[0])
        if scriptdir == 'DATA':
            studydir, casedir = os.path.split(topdir)
            studydir = os.path.split(studydir)[1]
        else:
            casedir = ''
            studydir = scriptdir

        job_name = studydir.lower()
        if casedir:
            job_name += '_' + casedir.lower()

        job_lines = cs_batch.generate_header(job_name=job_name,
                                             package=self.package)

        if job_lines:
            if force_rebuild or not 'job_header' in self.sections[resource_name]:
                self.sections[resource_name]['job_header'] \
                    = os.linesep.join(job_lines)
        else:
            for k in ('n_procs', 'n_threads'):
                if not k in self.sections[resource_name]:
                    self.sections[resource_name][k] = ''

        return

    #---------------------------------------------------------------------------

    def __strip_extra_lines__(self, value):
        """
        Remove start and end empty lines
        """

        br = os.linesep
        lbr = len(br)

        while value[:lbr] == br:
            value = value[lbr:]
        while value[-lbr:] == br:
            value = value[:-lbr]

        return value

    #---------------------------------------------------------------------------

    def __parse__(self, lines):
        """
        Parse the file.
        """

        br = os.linesep
        lbr = len(br)

        sections = OrderedDict()
        comments = {}

        section_name = 'job_defaults'
        section_dict = OrderedDict()
        comment_dict = {}
        sub_section = False
        key_indent = -1  # not inside key if < 0
        key = ''
        value = ''
        comment = ''

        for line in lines:

            if sub_section:
                l = line.lstrip()
                if l[:1] == '[' and l.find(']') > 0:
                    sub_section = False
                    section_dict[key] = self.__strip_extra_lines__(value)
                    key = ''
                    value = ''
                else:
                    value += br + line
                    continue

            # determine indent and effective content
            l = line.lstrip()
            indent = len(line) - len(l)

            # handle multiline
            if key_indent > -1:
                if indent > key_indent or len(line) == 0:
                    if len(value) > 0:
                        value += br + l
                    else:
                        value += l
                    continue
                else:
                    key_indent = -1

            # ignore empty lines
            if not line:
                if comment != '':
                    comment += br
                continue

            if key_indent < 0 and key:
                section_dict[key] = self.__strip_extra_lines__(value)
                key = ''
                value = ''

            if l[:1] in ('#', ';'):  # comment line
                comment += line + br
                continue

            if l[:1] == '[':
                i = l.find(']')
                if i > 0:
                    # New section; save previous one
                    sections[section_name] = section_dict
                    comments[section_name] = comment_dict
                    section_name = l[1:i].strip().lower()
                    # Possible subsection
                    j = section_name.find(':')
                    if j > -1:
                        key = section_name[j+1:]
                        section_name = section_name[:j]
                        sub_section = True
                        value = ''
                    if section_name in sections:
                        section_dict = sections[section_name]
                        comment_dict = comments[section_name]
                    else:
                        section_dict = OrderedDict()
                        comment_dict = {}
                    if comment != '':
                        if '' in comment_dict:
                            comment_dict[''] += br + comment.rstrip()
                        else:
                            comment_dict[''] = comment.rstrip()
                        comment = ''
            else:
                for i, c in enumerate(l):
                    if c in (':', '='):
                        key = l[:i].rstrip().lower()
                        value = l[i+1:].lstrip()
                        key_indent = indent
                        if comment != '':
                            comment_dict[key] = comment.rstrip()
                            comment = ''
                        break

        if sub_section or key:
            section_dict[key] = self.__strip_extra_lines__(value)

        sections[section_name] = section_dict
        comments[section_name] = comment_dict

        self.sections = sections
        self.comments = comments

    #---------------------------------------------------------------------------

    def __rebuild_lines__(self):
        """
        Update lines based on the current dictionnary values.
        """

        br = os.linesep
        lbr = len(br)

        # The following keys will use multiline sections by default
        multiline_section_keys = ('job_header',
                                  'run_prologue', 'run_epilogue',
                                  'compute_prologue', 'compute_epilogue')

        # Default sections we prefer to place first
        default_sections = ['setup', 'job_defaults', 'run']

        other_sections = []
        for sn in self.sections:
            if sn not in default_sections:
                other_sections.append(sn)
        for sn in self.comments:
            if sn not in default_sections:
                other_sections.append(sn)
        other_sections = list(set(other_sections))
        other_sections.sort()

        section_queue = default_sections + other_sections

        lines = []

        prev_is_multiline = False

        # now loop on sections

        for sn in section_queue:

            sn_comment = None
            if sn in self.comments:
                if '' in self.comments[sn]:
                    sn_comment = self.comments[sn][''].split(br)
                    if prev_is_multiline:
                        lines.append('[]')
                        lines.append('')
                        prev_is_multiline = False
                    lines += sn_comment
                    lines.append('')

            keys = []
            if sn in self.sections:
                for k in self.sections[sn]:
                    if self.sections[sn][k] != None:
                        keys.append(k)
            if sn in self.comments:
                for k in self.comments[sn]:
                    if self.comments[sn][k] != None:
                        keys.append(k)

            keys = list(set(keys))
            keys.sort()

            if sn == 'run':
                for k in ('stage', 'initialize', 'compute', 'finalize'):
                    if k in keys:
                        keys.remove(k)
                        keys.append(k)

            # Loop on keys with at least a value or comment

            simple_keys = []
            multiline_keys = []

            for k in keys:
                if k == '':
                    continue
                if k in multiline_section_keys:
                    multiline_keys.append(k)
                else:
                    simple_keys.append(k)

            has_simple = False

            if len(simple_keys) > 0 or sn_comment != None:
                lines.append('[' + sn.lower() + ']')
                lines.append('')
                prev_is_multiline = False
                has_simple = True

            k_value_l = 0

            # Standard keys first

            for k in simple_keys:

                k_comment = None
                k_value = None
                k_value_l = 0

                if sn in self.comments:
                    if k in self.comments[sn]:
                        k_comment = self.comments[sn][k].split(br)
                if sn in self.sections:
                    if k in self.sections[sn]:
                        k_value = str(self.sections[sn][k]).split(br)
                        k_value_l = len(k_value)

                if k_comment:
                    lines += k_comment
                if k_value_l > 0:
                    if k_value_l == 1:
                        v = k_value[0]
                        if v in ('True', 'False'):
                            v = v.lower()
                        lines.append(k + ': ' + v)
                    else:
                        if k_comment:
                            lines.append('')
                        lines.append(k + ': ')
                        for l in k_value:
                            lines.append('    ' + l)
                        lines.append('')
                elif k_comment:
                    lines.append(k + ': ')

            if has_simple:
                if k_value_l < 2:
                    lines.append('')

            # Multiline keys next

            for k in multiline_keys:

                k_comment = None
                k_value = None

                if sn in self.comments:
                    if k in self.comments[sn]:
                        k_comment = self.comments[sn][k].split(br)
                if sn in self.sections:
                    if k in self.sections[sn]:
                        k_value = str(self.sections[sn][k]).split(br)

                if k_comment:
                    if prev_is_multiline:
                        lines.append('[]')
                        lines.append('')
                    lines += k_comment
                    lines.append('')

                lines.append('[{0}:{1}]'.format(sn, k))
                lines.append('')

                if k_value:
                    lines += k_value
                    lines.append('')

                prev_is_multiline = True

        # End of loop on sections

        return lines

    #---------------------------------------------------------------------------

    def get(self, section, key):
        """
        Get a section:value combination (as a string, or None if not present)
        """

        value = None
        if section in self.sections:
            if key in self.sections[section]:
                value = str(self.sections[section][key])

        return value

    #---------------------------------------------------------------------------

    def get_bool(self, section, key):
        """
        Get a section:value combination as a boolean type, or None if not present
        """

        value = None
        if section in self.sections:
            if key in self.sections[section]:
                value = str(self.sections[section][key])

        if value != None:
            if value in ('0', 'f', 'false', 'n', 'no'):
                value = False
            elif value in ('1', 't', 'true', 'y', 'yes'):
                value = True
            elif value == '':
                value = None
            else:
                if self.path:
                    err_str = 'In file: ' + self.path + ':' + os.linesep
                else:
                    err_str = 'In run_conf object (' + str(self) + '):' + os.linesep
                    err_str += '  ['+section+']' + os.linesep
                    err_str += '  '+key+': ' + str(value) + os.linesep
                    err_str += 'is not a boolean value.'
                    raise TypeError(err_str)

        return value

    #---------------------------------------------------------------------------

    def get_int(self, section, key):
        """
        Get a section:value combination as an integer, or None if not present
        """

        value = None
        if section in self.sections:
            if key in self.sections[section]:
                value = str(self.sections[section][key])

        if value:
            try:
                value = int(value)
            except Exception:
                if self.path:
                    err_str = 'In file: ' + self.path + ':' + os.linesep
                else:
                    err_str = 'In run_conf object (' + str(self) + '):' + os.linesep
                err_str += '  ['+section+']' + os.linesep
                err_str += '  '+key+': ' + str(value) + os.linesep
                err_str += 'is not an integer value.'
                raise TypeError(err_str)

        return value

    #---------------------------------------------------------------------------

    def set(self, section, key, value):
        """
        Set a section:key value combination
        """

        if not section in self.sections:
            self.sections[section] = OrderedDict()
        if value != None:
            self.sections[section][key] = str(value)
        else:
            self.sections[section][key] = None

    #---------------------------------------------------------------------------

    def save(self, path=None, new=False):
        """
        Save run configuration, optionally to a different path.
        """

        br = os.linesep

        if path:
            self.path = path
            if not new:
                self.lines = []
                try:
                    f = open(self.path, mode = 'r')
                    self.lines = f.readlines()
                    f.close()
                    for i in range(len(self.lines)):
                        self.lines[i] = self.lines[i].rstrip()
                except IOError:
                    pass

        lines = self.__rebuild_lines__()

        f = open(self.path, mode='w')
        for line in lines:
            f.write(line + br)
        f.close()

    #---------------------------------------------------------------------------

    def get_coupling_parameters(self):
        """
        Return the list of coupled domains defined inside a run.cfg.
        Empty list if single-case run.
        """

        domains = []

        domain_names = self.get('setup', 'coupled_domains')

        if domain_names:
            for dom in domain_names.split(":"):
                d = self.sections[dom.lower()]
                domains.append(OrderedDict())

                for key in d.keys():
                    if d[key] and d[key] != 'None':
                        if key in ('n_procs_max', 'n_procs_min', 'n_procs_weight'):
                            domains[-1][key] = int(d[key])
                        else:
                            domains[-1][key] = str(d[key])
                    else:
                        domains[-1][key] = None

        return domains


#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------



# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

"""
This module contains the following classes and function:
- ManageCasesView
"""

#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------

import logging, os

#-------------------------------------------------------------------------------
# Third-party modules
#-------------------------------------------------------------------------------

from code_saturne.Base.QtCore    import *
from code_saturne.Base.QtGui     import *
from code_saturne.Base.QtWidgets import *

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

from code_saturne.model.Common import LABEL_LENGTH_MAX, GuiParam
from code_saturne.Base.QtPage import RegExpValidator, from_qvariant, to_text_string
from code_saturne.studymanager_gui.ManageCasesForm import Ui_ManageCasesForm
from code_saturne.studymanager_gui.ManageCasesModel import ManageCasesModel

#-------------------------------------------------------------------------------
# log config
#-------------------------------------------------------------------------------

logging.basicConfig()
log = logging.getLogger("ManageCasesView")
log.setLevel(GuiParam.DEBUG)


#-------------------------------------------------------------------------------
# item class
#-------------------------------------------------------------------------------
class item_class(object):
    '''
    custom data object
    '''
    def __init__(self, idx, name, compute, post, status, run_id):
        self.index   = idx
        self.name    = name
        self.compute = compute
        self.post    = post
        self.status  = status
        self.run_id  = run_id

    def __repr__(self):
        return "case : %s // compute : %s // post %s // status %s // run_id %s"\
               % (self.name, self.compute, self.post, self.status, self.run_id)

#-------------------------------------------------------------------------------
# Treeitem class
#-------------------------------------------------------------------------------
class TreeItem(object):
    '''
    a python object used to return row/column data, and keep note of
    it's parents and/or children
    '''
    def __init__(self, item, header, parentItem):
        self.item = item
        self.parentItem = parentItem
        self.header = header
        self.childItems = []


    def appendChild(self, item):
        self.childItems.append(item)


    def child(self, row):
        return self.childItems[row]


    def childCount(self):
        return len(self.childItems)


    def columnCount(self):
        return 5


    def data(self, column, role):
        if self.item == None:
            if column == 0:
                return self.header
            else:
                return None
        else:
            if column == 0 and role == Qt.DisplayRole:
                return self.item.name
            elif column == 1 and role == Qt.CheckStateRole:
                value = self.item.status
                if value == 'on':
                    return Qt.Checked
                else:
                    return Qt.Unchecked
            elif column == 2 and role == Qt.CheckStateRole:
                value = self.item.compute
                if value == 'on':
                    return Qt.Checked
                else:
                    return Qt.Unchecked
            elif column == 3 and role == Qt.CheckStateRole:
                value = self.item.post
                if value == 'on':
                    return Qt.Checked
                else:
                    return Qt.Unchecked
            elif column == 4 and role == Qt.DisplayRole:
                return self.item.run_id
        return None


    def parent(self):
        return self.parentItem


    def row(self):
        if self.parentItem:
            return self.parentItem.childItems.index(self)
        return 0


#-------------------------------------------------------------------------------
#
#-------------------------------------------------------------------------------

class LabelDelegate(QItemDelegate):
    """
    """
    def __init__(self, parent=None, xml_model=None):
        super(LabelDelegate, self).__init__(parent)
        self.parent = parent
        self.mdl = xml_model


    def createEditor(self, parent, option, index):
        editor = QLineEdit(parent)
        rx = "[\-_A-Za-z0-9]{1," + str(LABEL_LENGTH_MAX) + "}"
        self.regExp = QRegExp(rx)
        v =  RegExpValidator(editor, self.regExp)
        editor.setValidator(v)
        return editor


    def setEditorData(self, editor, index):
        editor.setAutoFillBackground(True)
        v = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
        self.p_value = str(v)
        editor.setText(v)

    def setModelData(self, editor, model, index):
        if not editor.isModified():
            return

        if editor.validator().state == QValidator.Acceptable:
            p_value = str(editor.text())
            model.setData(index, p_value, Qt.DisplayRole)


#-------------------------------------------------------------------------------
# StandarItemModelOutput class
#-------------------------------------------------------------------------------

class CaseStandardItemModel(QAbstractItemModel):

    def __init__(self, parent, case, mdl):
        """
        """
        QAbstractItemModel.__init__(self)

        self.parent = parent
        self.case   = case
        self.mdl    = mdl

        self.noderoot = {}
        self.prtlist = []
        for name in self.mdl.StudyList:
            self.prtlist.append(name)

        self.rootItem = TreeItem(None, "ALL", None)
        self.parents = {0 : self.rootItem}

        self.disabledItem = []
        self.populateModel()


    def columnCount(self, parent = None):
        if parent and parent.isValid():
            return parent.internalPointer().columnCount()
        else:
            return 5


    def data(self, index, role):
        if not index.isValid():
            return None

        item = index.internalPointer()

        # ToolTips
        if role == Qt.ToolTipRole:
            return None

        # StatusTips
        if role == Qt.StatusTipRole:
            if index.column() == 0:
                return self.tr("Case name")
            elif index.column() == 1:
                return self.tr("Status")
            elif index.column() == 2:
                return self.tr("Compute")
            elif index.column() == 3:
                return self.tr("Post-processing")
            elif index.column() == 4:
                return self.tr("Run_id")

        # Display
        if role == Qt.DisplayRole:
            return item.data(index.column(), role)
        elif role == Qt.CheckStateRole:
            return item.data(index.column(), role)

        return None


    def flags(self, index):
        if not index.isValid():
            return Qt.ItemIsEnabled

        # disable item
        if (index.row(), index.column()) in self.disabledItem:
            return Qt.ItemIsEnabled

        itm = index.internalPointer()
        if index.column() == 1 or index.column() == 2 or index.column() == 3:
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsUserCheckable
        elif index.column() == 0:
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable
        else:
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable


    def headerData(self, section, orientation, role):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            if section == 0:
                return self.tr("Case name")
            elif section == 1:
                return self.tr("Status")
            elif section == 2:
                return self.tr("Compute")
            elif section == 3:
                return self.tr("Post-\nprocessing")
            elif section == 4:
                return self.tr("run_id")
        return None


    def index(self, row, column, parent = QModelIndex()):
        if not self.hasIndex(row, column, parent):
            return QModelIndex()

        if not parent.isValid():
            parentItem = self.rootItem
        else:
            parentItem = parent.internalPointer()

        try:
            childItem = parentItem.child(row)
        except:
            childItem = None

        if childItem:
            return self.createIndex(row, column, childItem)
        else:
            return QModelIndex()


    def parent(self, index):
        if not index.isValid():
            return QModelIndex()

        childItem = index.internalPointer()
        if not childItem:
            return QModelIndex()

        parentItem = childItem.parent()

        if parentItem == self.rootItem:
            return QModelIndex()

        return self.createIndex(parentItem.row(), 0, parentItem)


    def rowCount(self, parent=QModelIndex()):
        if parent.column() > 0:
            return 0
        if not parent.isValid():
            p_Item = self.rootItem
        else:
            p_Item = parent.internalPointer()
        return p_Item.childCount()


    def populateModel(self):
        for name in self.prtlist:
            row = self.rowCount()
            status  = self.mdl.getStudyStatus(name)
            item = item_class(-1, name, "off", "off", status, "")
            newparent = TreeItem(item, name, self.rootItem)
            self.rootItem.appendChild(newparent)
            self.noderoot[name] = newparent
#            self.disabledItem.append((row, 2))
#            self.disabledItem.append((row, 3))
#            self.disabledItem.append((row, 4))

        for name in self.prtlist:
            for idx in self.mdl.getCaseList(name):
                parentItem = self.noderoot[name]
                cname = self.mdl.getCaseName(name, idx)
                compute = self.mdl.getComputeStatus(name, idx)
                post    = self.mdl.getPostStatus(name, idx)
                status  = self.mdl.getStatus(name, idx)
                run_id  = self.mdl.getRunId(name, idx)
                item = item_class(idx, cname, compute, post, status, run_id)
                new_item = TreeItem(item, "", parentItem)
                parentItem.appendChild(new_item)


    def setData(self, index, value, role=None):
        item = index.internalPointer()

        if index.column() == 1:
            v = from_qvariant(value, int)
            if v == Qt.Checked:
                item.item.status = "on"
            else:
                item.item.status = "off"
            if item not in self.noderoot.values():
                itm = item.parentItem
                self.mdl.setStatus(itm.item.name, item.item.index, item.item.status)
            else:
                self.mdl.setStudyStatus(item.item.name, item.item.status)

        elif index.column() == 2:
            v = from_qvariant(value, int)
            if v == Qt.Checked:
                item.item.compute = "on"
            else:
                item.item.compute = "off"
            if item not in self.noderoot.values():
                itm = item.parentItem
                self.mdl.setComputeStatus(itm.item.name, item.item.index, item.item.compute)

        elif index.column() == 3:
            v = from_qvariant(value, int)
            if v == Qt.Checked:
                item.item.post = "on"
            else:
                item.item.post = "off"
            if item not in self.noderoot.values():
                itm = item.parentItem
                self.mdl.setPostStatus(itm.item.name, item.item.index, item.item.post)

        elif index.column() == 4:
            run_id = str(from_qvariant(value, to_text_string))
            item.item.run_id = run_id
            if item not in self.noderoot.values():
                itm = item.parentItem
                self.mdl.setRunId(itm.item.name, item.item.index, item.item.run_id)

        self.dataChanged.emit(QModelIndex(), QModelIndex())

        return True


#-------------------------------------------------------------------------------
# Main class
#-------------------------------------------------------------------------------

class ManageCasesView(QWidget, Ui_ManageCasesForm):
    """
    """
    def __init__(self, parent, case):
        """
        Constructor
        """
        QWidget.__init__(self, parent)

        Ui_ManageCasesForm.__init__(self)
        self.setupUi(self)

        self.case = case
        self.parent = parent
        self.mdl = ManageCasesModel(self.case)

        if not self.mdl.list_study:
            self.add_study()

        self.modelCases = CaseStandardItemModel(self.parent, self.case, self.mdl)
        self.treeViewCases.setModel(self.modelCases)
        self.treeViewCases.setAlternatingRowColors(True)
        self.treeViewCases.setSelectionBehavior(QAbstractItemView.SelectItems)
        self.treeViewCases.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.treeViewCases.setEditTriggers(QAbstractItemView.DoubleClicked)
        self.treeViewCases.expandAll()
        self.treeViewCases.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.treeViewCases.setDragEnabled(False)

        runidDelegate = LabelDelegate(self.treeViewCases, self.mdl)
        self.treeViewCases.setItemDelegateForColumn(4, runidDelegate)

        self.treeViewCases.resizeColumnToContents(0)

        self.pushButtonAdd.clicked.connect(self.slotAddCase)
        self.pushButtonDelete.clicked.connect(self.slotDeleteCase)
        self.pushButtonAddStudy.clicked.connect(self.slotAddStudy)
        self.pushButtonDeleteStudy.clicked.connect(self.slotDeleteStudy)
        self.toolButtonDuplicate.clicked.connect(self.slotDuplicateCase)
        self.treeViewCases.clicked.connect(self.slotChangeSelection)

        self.checkBoxPrepro.clicked.connect(self.slotPreproStatus)
        self.checkBoxPost.clicked.connect(self.slotPostStatus)
        self.checkBoxCompare.clicked.connect(self.slotCompareStatus)
        self.pushButtonPrepro.clicked.connect(self.slotPreproFile)
        self.pushButtonPost.clicked.connect(self.slotPostFile)
        self.pushButtonInput.clicked.connect(self.slotInputFile)
        self.lineEditPreproArgs.textChanged[str].connect(self.slotPreproArgs)
        self.lineEditPostArgs.textChanged[str].connect(self.slotPostArgs)
        self.lineEditCompareArgs.textChanged[str].connect(self.slotCompareArgs)

        self.groupBoxPrepro.hide()
        self.groupBoxPost.hide()
        self.groupBoxInput.hide()
        self.groupBoxCompare.hide()

        self.lineEditPrepro.setEnabled(False)
        self.lineEditInput.setEnabled(False)
        self.lineEditPost.setEnabled(False)
        self.pushButtonDelete.setEnabled(False)
        self.pushButtonDeleteStudy.setEnabled(False)
        self.toolButtonDuplicate.setEnabled(False)
        self.pushButtonAdd.setEnabled(False)
    #TODO ajouter self.disabledItem.append((row, 3)) pour les noeuds study


    def add_case(self, study):
        """
        public slot
        """
        title = self.tr("Add existing case")

        cur_path = os.getcwd()
        path = os.path.abspath(os.path.join(self.mdl.repo, study))
        os.chdir(path)

        dialog = QFileDialog()
        dialog.setWindowTitle(title)
        dialog.setDirectory(path)
        dialog.setFileMode(QFileDialog.DirectoryOnly)

        if dialog.exec_() == 1:

            s = dialog.selectedFiles()
            dir_path = str(s[0])
            dir_path = os.path.relpath(dir_path)
            if dir_path not in os.listdir(path):
                title = self.tr("WARNING")
                msg   = self.tr("This selected case is not in the directory of the study")
                QMessageBox.information(self, title, msg)
            else:
                self.mdl.addCase(study, dir_path)

            log.debug("add_case -> %s" % dir_path)
        os.chdir(cur_path)


    def add_study(self):
        """
        public slot
        """
        title = self.tr("Add existing study")

        cur_path = os.getcwd()
        path = os.path.abspath(self.mdl.repo)
        os.chdir(path)

        dialog = QFileDialog()
        dialog.setWindowTitle(title)
        dialog.setDirectory(path)
        dialog.setFileMode(QFileDialog.DirectoryOnly)

        if dialog.exec_() == 1:

            s = dialog.selectedFiles()
            dir_path = str(s[0])
            dir_path = os.path.relpath(dir_path)
            if dir_path not in os.listdir(path):
                title = self.tr("WARNING")
                msg   = self.tr("This selected study is not in the repository directory")
                QMessageBox.information(self, title, msg)
            else:
                self.mdl.addStudy(dir_path)

                title = self.tr("Loading study/case")
                msg   = self.tr("Do you want to use automatic load ?")

                reply = QMessageBox.question(self,
                                             title,
                                             msg,
                                             QMessageBox.Yes|
                                             QMessageBox.No)
                if reply == QMessageBox.Yes:
                    self.mdl.loadCases(dir_path)

            log.debug("add_study -> %s" % dir_path)
        os.chdir(cur_path)


    def slotAddStudy(self):
        """
        public slot
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        self.add_study()
        self.modelCases = CaseStandardItemModel(self.parent, self.case, self.mdl)
        self.treeViewCases.setModel(self.modelCases)
        self.groupBoxPrepro.hide()
        self.groupBoxPost.hide()
        self.groupBoxInput.hide()
        self.treeViewCases.expandAll()
        self.slotChangeSelection()


    def slotDeleteStudy(self):
        """
        public slot
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        study = current.internalPointer().item.name
        self.mdl.deleteStudy(study)
        self.modelCases = CaseStandardItemModel(self.parent, self.case, self.mdl)
        self.treeViewCases.setModel(self.modelCases)
        self.groupBoxPrepro.hide()
        self.groupBoxPost.hide()
        self.groupBoxInput.hide()
        self.treeViewCases.expandAll()
        self.slotChangeSelection()


    def slotAddCase(self):
        """
        public slot
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        study = ""
        if current.parent() == self.treeViewCases.rootIndex():
            study = current.internalPointer().item.name
        else:
            study = current.parent().internalPointer().item.name
        self.add_case(study)
        self.modelCases = CaseStandardItemModel(self.parent, self.case, self.mdl)
        self.treeViewCases.setModel(self.modelCases)
        self.groupBoxPrepro.hide()
        self.groupBoxPost.hide()
        self.groupBoxInput.hide()
        self.treeViewCases.expandAll()
        self.slotChangeSelection()


    def slotDeleteCase(self):
        """
        public slot
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        study = current.parent().internalPointer().item.name
        self.mdl.deleteCase(study, idx)
        self.modelCases = CaseStandardItemModel(self.parent, self.case, self.mdl)
        self.treeViewCases.setModel(self.modelCases)
        self.groupBoxPrepro.hide()
        self.groupBoxPost.hide()
        self.groupBoxInput.hide()
        self.treeViewCases.expandAll()
        self.slotChangeSelection()


    def slotDuplicateCase(self):
        """
        public slot
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        study = current.parent().internalPointer().item.name
        self.mdl.duplicateCase(study, idx)
        self.modelCases = CaseStandardItemModel(self.parent, self.case, self.mdl)
        self.treeViewCases.setModel(self.modelCases)
        self.groupBoxPrepro.hide()
        self.groupBoxPost.hide()
        self.groupBoxInput.hide()
        self.treeViewCases.expandAll()
        self.slotChangeSelection()


    def slotChangeSelection(self):
        """
        """
        self.pushButtonDelete.setEnabled(False)
        self.pushButtonDeleteStudy.setEnabled(False)
        self.toolButtonDuplicate.setEnabled(False)
        self.pushButtonAdd.setEnabled(True)
        self.groupBoxPrepro.hide()
        self.groupBoxPost.hide()
        self.groupBoxInput.hide()
        self.groupBoxCompare.hide()

        current = self.treeViewCases.currentIndex()
        idx = current.row()
        if current == self.treeViewCases.rootIndex():
            self.pushButtonAdd.setEnabled(False)
        elif current.parent() == self.treeViewCases.rootIndex():
            # study
            self.pushButtonDeleteStudy.setEnabled(True)
            self.groupBoxPost.show()
            study = current.internalPointer().item.name

            status = self.mdl.getStudyPostScriptStatus(study)
            if status == "on":
                self.checkBoxPost.setChecked(True)
            else:
                self.checkBoxPost.setChecked(False)

            script_name = self.mdl.getStudyPostScriptName(study)
            self.lineEditPost.setText(str(script_name))
            if status == "on":
                if script_name != "":
                    self.pushButtonPost.setStyleSheet("background-color: green")
                else:
                    self.pushButtonPost.setStyleSheet("background-color: red")
            else:
                self.pushButtonPost.setStyleSheet("background-color: None")

            script_args = self.mdl.getStudyPostScriptArgs(study)
            self.lineEditPostArgs.setText(str(script_args))
        else:
            # case
            self.pushButtonDelete.setEnabled(True)
            self.toolButtonDuplicate.setEnabled(True)
            self.groupBoxPrepro.show()
            self.groupBoxPost.show()
            self.groupBoxInput.show()
            self.groupBoxCompare.show()
            study = current.parent().internalPointer().item.name

            # prepro
            status = self.mdl.getPreproScriptStatus(study, idx)
            if status == "on":
                self.checkBoxPrepro.setChecked(True)
            else:
                self.checkBoxPrepro.setChecked(False)

            script_name = self.mdl.getPreproScriptName(study, idx)
            self.lineEditPrepro.setText(str(script_name))
            if status == "on":
                if script_name != "":
                    self.pushButtonPrepro.setStyleSheet("background-color: green")
                else:
                    self.pushButtonPrepro.setStyleSheet("background-color: red")
            else:
                self.pushButtonPrepro.setStyleSheet("background-color: None")

            script_args = self.mdl.getPreproScriptArgs(study, idx)
            self.lineEditPreproArgs.setText(str(script_args))

            # post
            status = self.mdl.getPostScriptStatus(study, idx)
            if status == "on":
                self.checkBoxPost.setChecked(True)
            else:
                self.checkBoxPost.setChecked(False)

            script_name = self.mdl.getPostScriptName(study, idx)
            self.lineEditPost.setText(str(script_name))
            if status == "on":
                if script_name != "":
                    self.pushButtonPost.setStyleSheet("background-color: green")
                else:
                    self.pushButtonPost.setStyleSheet("background-color: red")
            else:
                self.pushButtonPost.setStyleSheet("background-color: None")

            script_args = self.mdl.getPostScriptArgs(study, idx)
            self.lineEditPostArgs.setText(str(script_args))

            input_name = self.mdl.getPostScriptInput(study, idx)
            self.lineEditInput.setText(str(input_name))
            if input_name == '<multiple inputs>':
                self.pushButtonInput.setEnabled(False)
            else:
                self.pushButtonInput.setEnabled(True)

            # compare
            status = self.mdl.getCompareStatus(study, idx)
            if status == "on":
                self.checkBoxCompare.setChecked(True)
            else:
                self.checkBoxCompare.setChecked(False)
            compare_args = self.mdl.getCompareArgs(study, idx)
            self.lineEditCompareArgs.setText(str(compare_args))


    @pyqtSlot()
    def slotPreproStatus(self):
        """
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        study = current.parent().internalPointer().item.name
        if self.checkBoxPrepro.isChecked():
            self.mdl.setPreproScriptStatus(study, idx, "on")
            if self.mdl.getPreproScriptName(study, idx) != "":
                self.pushButtonPrepro.setStyleSheet("background-color: green")
            else:
                self.pushButtonPrepro.setStyleSheet("background-color: red")
        else:
            self.mdl.setPreproScriptStatus(study, idx, "off")
            self.pushButtonPrepro.setStyleSheet("background-color: None")


    @pyqtSlot()
    def slotCompareStatus(self):
        """
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        study = current.parent().internalPointer().item.name
        if self.checkBoxCompare.isChecked():
            self.mdl.setCompareStatus(study, idx, "on")
        else:
            self.mdl.setCompareStatus(study, idx, "off")


    @pyqtSlot()
    def slotPostStatus(self):
        """
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        if current.parent() == self.treeViewCases.rootIndex():
            study = current.internalPointer().item.name
            if self.checkBoxPost.isChecked():
                self.mdl.setStudyPostScriptStatus(study, "on")
                if self.mdl.getStudyPostScriptName(study) != "":
                    self.pushButtonPost.setStyleSheet("background-color: green")
                else:
                    self.pushButtonPost.setStyleSheet("background-color: red")
            else:
                self.mdl.setStudyPostScriptStatus(study, "off")
                self.pushButtonPost.setStyleSheet("background-color: None")
        else:
            study = current.parent().internalPointer().item.name
            if self.checkBoxPost.isChecked():
                self.mdl.setPostScriptStatus(study, idx, "on")
                if self.mdl.getPostScriptName(study, idx) != "":
                    self.pushButtonPost.setStyleSheet("background-color: green")
                else:
                    self.pushButtonPost.setStyleSheet("background-color: red")
            else:
                self.mdl.setPostScriptStatus(study, idx, "off")
                self.pushButtonPost.setStyleSheet("background-color: None")


    @pyqtSlot(str)
    def slotPreproArgs(self, text):
        """
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        study = current.parent().internalPointer().item.name
        args = str(text)
        self.mdl.setPreproScriptArgs(study, idx, args)


    @pyqtSlot(str)
    def slotCompareArgs(self, text):
        """
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        study = current.parent().internalPointer().item.name
        args = str(text)
        self.mdl.setCompareArgs(study, idx, args)


    @pyqtSlot(str)
    def slotPostArgs(self, text):
        """
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        args = str(text)
        if current.parent() == self.treeViewCases.rootIndex():
            study = current.internalPointer().item.name
            self.mdl.setStudyPostScriptArgs(study, args)
        else:
            study = current.parent().internalPointer().item.name
            self.mdl.setPostScriptArgs(study, idx, args)


    def slotPreproFile(self):
        """
        Select a prepro script
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        study = current.parent().internalPointer().item.name

        cur_path = os.getcwd()
        base_dir = os.path.abspath(os.path.join(self.mdl.repo, study))
        rep = os.path.abspath(os.path.join(base_dir, "MESH"))
        if not os.path.isdir(rep):
            rep = os.path.abspath(os.path.join(os.path.split(base_dir)[0], "MESH"))
        if not os.path.isdir(rep):
            rep = base_dir
        title = self.tr("preprocess script")
        filetypes = self.tr("(*py*);;All Files (*)")
        file = QFileDialog.getOpenFileName(self, title, rep, filetypes)[0]
        file = str(file)

        if not file:
            return
        file = os.path.basename(file)

        if file not in os.listdir(rep):
            title = self.tr("WARNING")
            msg   = self.tr("This selected file is not in the MESH directory of te study")
            QMessageBox.information(self, title, msg)
        else:
            self.lineEditPrepro.setText(str(file))
            self.mdl.setPreproScriptName(study, idx, file)


    def slotPostFile(self):
        """
        public slot
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        study = ""
        if current.parent() == self.treeViewCases.rootIndex():
            study = current.internalPointer().item.name
        else:
            study = current.parent().internalPointer().item.name

        cur_path = os.getcwd()
        base_dir = os.path.abspath(os.path.join(self.mdl.repo, study))
        rep = os.path.abspath(os.path.join(base_dir, "POST"))
        if not os.path.isdir(rep):
            rep = os.path.abspath(os.path.join(os.path.split(base_dir)[0], "POST"))
        if not os.path.isdir(rep):
            rep = base_dir
        title = self.tr("postprocess script")
        filetypes = self.tr("All Files (*)")
        file = QFileDialog.getOpenFileName(self, title, rep, filetypes)[0]
        file = str(file)

        if not file:
            return
        file = os.path.basename(file)
        if file not in os.listdir(rep):
            title = self.tr("WARNING")
            msg   = self.tr("This selected file is not in the POST directory of the study")
            QMessageBox.information(self, title, msg)
        else:
            self.lineEditPost.setText(str(file))
            if current.parent() == self.treeViewCases.rootIndex():
                self.mdl.setStudyPostScriptName(study, file)
            else:
                self.mdl.setPostScriptName(study, idx, file)
        return


    def slotInputFile(self):
        """
        public slot
        """
        current = self.treeViewCases.currentIndex()
        idx = current.row()
        study = current.parent().internalPointer().item.name

        cur_path = os.getcwd()
        base_dir = os.path.abspath(os.path.join(self.mdl.repo, study))
        rep = os.path.abspath(os.path.join(base_dir, "POST"))
        if not os.path.isdir(rep):
            rep = os.path.abspath(os.path.join(os.path.split(base_dir)[0], "POST"))
        if not os.path.isdir(rep):
            rep = base_dir
        title = self.tr("input file for postprocess script")
        filetypes = self.tr("All Files (*)")
        file = QFileDialog.getOpenFileName(self, title, rep, filetypes)[0]
        file = str(file)

        if not file:
            return
        file = os.path.basename(file)
        if file not in os.listdir(rep):
            title = self.tr("WARNING")
            msg   = self.tr("This selected file is not in the POST directory of the study")
            QMessageBox.information(self, title, msg)
        else:
            self.lineEditInput.setText(str(file))
            self.mdl.setPostScriptInput(study, idx, file)


#-------------------------------------------------------------------------------
# Testing part
#-------------------------------------------------------------------------------


if __name__ == "__main__":
    pass


#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------



# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

import sys, unittest
from code_saturne.model.XMLvariables import Model
from code_saturne.model.XMLengine import *
from code_saturne.model.XMLmodel import *
from code_saturne.model.NonCondensableModel import NonCondensableModel
import copy

#-------------------------------------------------------------------------------
# Constructor
#-------------------------------------------------------------------------------


class DropletCondensationEvaporationModel(NonCondensableModel, Variables, Model):
    """
    This class manages the wall tranfer model objects in the XML file
    """


    def __init__(self, case):
        """
        Constuctor.
        """
        #
        # XML file parameters
        NonCondensableModel.__init__(self, case)
        self.case = case
        self.XMLNodeclosure = self.case.xmlGetNode('closure_modeling')
        self.XMLMassTrans   = self.XMLNodeclosure.xmlInitNode('mass_transfer_model')


    def defaultValues(self):
        default = {}
        default['yplusmodel']               = "Yplus_value"
        default['yplusvalue']               = 100.

        return default


    @Variables.noUndo
    def getYPlusModel(self):
        """
        get Y+ status
        """
        ChildNode = self.XMLMassTrans.xmlGetNode('yplusmodel')
        if ChildNode == None:
           model = self.defaultValues()['yplusmodel']
           self.setYPlusModel(model)
        model = self.XMLMassTrans.xmlGetNode('yplusmodel')['model']
        return model


    @Variables.undoLocal
    def setYPlusModel(self, model):
        """
        put Y+ status
        """
        self.isInList(model, ('center','Yplus_value','diameter'))

        ChildNode = self.XMLMassTrans.xmlInitChildNode('yplusmodel')
        ChildNode.xmlSetAttribute(model = model)

        if model != "Yplus_value" :
           self.XMLMassTrans.xmlRemoveChild('yplusvalue')


    @Variables.noUndo
    def getYPlusValue(self):
        """
        get value for Y Plus
        """
        value = self.XMLMassTrans.xmlGetDouble('yplusvalue')
        if value == None :
           value = self.defaultValues()['yplusvalue']
           self.setYPlusValue(value)
        return value

    @Variables.undoLocal
    def setYPlusValue(self, value):
        """
        put value for Y Plus
        """
        self.isGreater(value, 0.)
        self.XMLMassTrans.xmlSetData('yplusvalue', value)

    def resetToDefaultValues(self):
        default = self.defaultValues()
        self.setYPlusModel(default["yplusmodel"])
        self.setYPlusValue(default["yplusvalue"])


#-------------------------------------------------------------------------------
# DropletCondensationEvaporation test case
#-------------------------------------------------------------------------------

class DropletCondensationEvaporationTestCase(ModelTest):
    """
    """
    def checkDropletCondensationEvaporationInstantiation(self):
        """Check whether the DropletCondensationEvaporation class could be instantiated"""
        model = None
        model = DropletCondensationEvaporationModel(self.case)
        assert model != None, 'Could not instantiate DropletCondensationEvaporationModel'


    def checkGetandSetYPlusModel(self):
        """Check whether the WallTransferModel class could set and get YPlusModel"""
        mdl = DropletCondensationEvaporationModel(self.case)
        mdl.setYPlusModel('Yplus_value')
        doc = '''<mass_transfer_model>
                         <nucleate_boiling/>
                         <yplusmodel model="Yplus_value"/>
                 </mass_transfer_model>'''
        assert mdl.XMLMassTrans == self.xmlNodeFromString(doc),\
            'Could not set YPlusModel'
        assert mdl.getYPlusModel() == 'Yplus_value',\
            'Could not get YPlusModel'


    def checkGetandSetYPlusValue(self):
        """Check whether the WallTransferModel class could set and get YPlusValue"""
        mdl = DropletCondensationEvaporationModel(self.case)
        mdl.setYPlusModel('Yplus_value')
        mdl.setYPlusValue(260)
        doc = '''<mass_transfer_model>
                         <nucleate_boiling/>
                         <yplusmodel model="Yplus_value"/>
                         <yplusvalue>
                                 260
                         </yplusvalue>
                 </mass_transfer_model>'''
        assert mdl.XMLMassTrans == self.xmlNodeFromString(doc),\
            'Could not set YPlusValue'
        assert mdl.getYPlusValue() == 260,\
            'Could not get YPlusValue'


def suite():
    testSuite = unittest.makeSuite(WallTransferTestCase, "check")
    return testSuite

def runTest():
    print("WallTransferTestCase")
    runner = unittest.TextTestRunner()
    runner.run(suite())



# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------

import string
import unittest

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

from code_saturne.model.XMLvariables import Model, Variables
from code_saturne.model.XMLmodel     import ModelTest
from code_saturne.model.XMLengine import *
from code_saturne.model.TurbulenceNeptuneModel import TurbulenceModelsDescription

class NumericalParamEquatModel(Model):

    """
    This class manages the turbulence objects in the XML file
    """

    def __init__(self, case) :
        """
        Constuctor.
        """
        #
        # XML file parameters
        self.case            = case
        self.XMLThermo       = self.case.xmlGetNode('thermophysical_models')
        self.XMLNodeVariable = self.XMLThermo.xmlGetNode('variables')
        self.XMLClosure      = self.case.xmlGetNode('closure_modeling')
        self.XMLTurbulence   = self.XMLClosure.xmlInitNode('turbulence')
        self.XMLTurbVariable = self.XMLTurbulence.xmlInitNode('variables')
        self.XMLAddScalar    = self.case.xmlInitNode('additional_scalars')
        self.XMLScalar       = self.XMLAddScalar.xmlInitNode('scalars')
        self.XMLnonCond      = self.XMLThermo.xmlInitNode('non_condensable_list')

        self.imgra_model = ['iterative_handling', \
                            'sweep_linear', \
                            'sweep_linear_ext_neigh', \
                            'least_square_linear', \
                            'least_square_linear_ext_neigh', \
                            'iterative_handling_weighted', \
                            'least_square_linear_weighted', \
                            'least_square_linear_ext_neigh_weighted']


    def defaultValues(self) :
        default = {}

        default['max_iter_number']                  = 10000
        default['solver_precision']                 = 1e-5
        default['solver_precision_alpha']           = 1e-8
        default['solver_precision_enthal']          = 1e-7
        default['slope_test']                       = 'on'
        default['flux_reconstruction']              = 'off'
        default['order_scheme']                     = 'solu'
        default['order_scheme_pressure_turbulence'] = 'upwind'
        default['solver']                           = 'automatic'
        default['gradient_reconstruction']          = 'sweep_linear_ext_neigh'
        return default


    def getVariableList(self) :
        """
        return list of variables
        """
        list = []
        from code_saturne.model.MainFieldsModel import MainFieldsModel
        for node in self.XMLNodeVariable.xmlGetNodeList('variable') :
            if self._isPressure(node) != 1 :
                # control to add enthalpy only if solved!!!
                if self._isEnthalpy(node) == 1 :
                    field = node['field_id']
                    if MainFieldsModel(self.case).getEnergyResolution(field) == "on":
                        list.append(node['label'])
                else:
                    list.append(node['label'])

        if self.XMLTurbVariable != None:
            for node in self.XMLTurbVariable.xmlGetNodeList('variable') :
                list.append(node['label'])

        for node in self.XMLScalar.xmlGetNodeList('variable') :
            list.append(node['label'])

        for node in self.XMLnonCond.xmlGetNodeList('variable') :
            list.append(node['label'])

        del MainFieldsModel
        return list


    def _isPressure(self, node) :
        """
        Return : 1 if name of node is 'pressure', 0 if not
        """
        if node and node['name'] == 'pressure':
            return 1
        else:
            return 0


    def _isTurbulenceVariable(self, node) :
        """
        Return : 1 if name of node is a turbulence variable, 0 if not
        """
        if node and node['name'] in TurbulenceModelsDescription.turbulenceVariables['all'] :
            return 1
        else:
            return 0


    def _isAlpha(self, node) :
        """
        Return : 1 if name of node is 'alpha', 0 if not
        """
        if node and node['name'] == 'volume_fraction':
            return 1
        else:
            return 0


    def _isEnthalpy(self, node):
        """
        Return : 1 if name of node is 'enthalpy', 0 if not
        """
        if node and node['name'] == 'enthalpy':
            return 1
        else:
            return 0


    def _getSchemeLabelNode(self, label):
        """ Private method: return node called with label'label' for scheme nodes"""
        for node in self.XMLNodeVariable.xmlGetNodeList('variable') :
            if node['label'] == label:
                if node['label'] == 'Pressure':
                    raise ValueError("This method does not run with pressure")
                else:
                    return node
        for node in self.XMLTurbVariable.xmlGetNodeList('variable') :
            if node['label'] == label:
                return node

        for node in self.XMLScalar.xmlGetNodeList('variable') :
            if node['label'] == label:
                return node

        for node in self.XMLnonCond.xmlGetNodeList('variable') :
            if node['label'] == label:
                return node

        raise ValueError("This label does not exist: " + label)


    @Variables.undoGlobal
    def setSchemeModel(self, label, value) :
        """
        Put value of order scheme for variable or scalar labelled label
        only if it 's different of default value
        """
        self.isInList(value, ('upwind', 'centered', 'solu'))
        node = self._getSchemeLabelNode(label)
        defval = self.defaultValues()['order_scheme']
        if (self._isTurbulenceVariable(node)) :
            defval = self.defaultValues()['order_scheme_pressure_turbulence']

        if value == defval :
            node.xmlRemoveChild('scheme')
        else:
            n = node.xmlInitNode('scheme')
            n['choice'] = value

        if value == "upwind":
            self.setSlopeTestStatus(label, "off")
        else:
            self.setSlopeTestStatus(label, "on")


    @Variables.noUndo
    def getSchemeModel(self, label) :
        """
        Return value of order scheme for variable labelled label
        """
        node = self._getSchemeLabelNode(label)
        value = self.defaultValues()['order_scheme']
        if (self._isTurbulenceVariable(node)) :
            value = self.defaultValues()['order_scheme_pressure_turbulence']
        n = node.xmlGetNode('scheme')
        if n:
            value = n['choice']
        return value


    @Variables.undoLocal
    def setSlopeTestStatus(self, label, status) :
        """
        Put status of slope test for variable labelled label
        """
        self.isOnOff(status)
        node = self._getSchemeLabelNode(label)
        if status == self.defaultValues()['slope_test']:
            node.xmlRemoveChild('slope_test')
        else:
            n = node.xmlInitNode('slope_test')
            n['status'] = status


    @Variables.noUndo
    def getSlopeTestStatus(self, label) :
        """
        Return value of slope test for variable labelled label
        """
        node = self._getSchemeLabelNode(label)
        value = self.defaultValues()['slope_test']
        n = node.xmlGetNode('slope_test')
        if n:
            value = n['status']
        return value


    @Variables.undoLocal
    def setSolverModel(self, label, value) :
        """
        Put value of solver for variable or scalar labelled label
        """
        self.isInList(value, ('automatic', 'jacobi', 'pcg', 'cgstab', 'jacobi_saturne', 'pcg_saturne', 'bicgstab_saturne', 'bicgstab2_saturne', 'gmres_saturne', 'gauss_seidel_saturne', 'sym_gauss_seidel_saturne', 'pcr3_saturne'))
        node = self._getSchemeLabelNode(label)
        default = self.defaultValues()['solver']
        if value == default:
            node.xmlRemoveChild('solver')
        else:
            n = node.xmlInitNode('solver')
            n['choice'] = value


    @Variables.noUndo
    def getSolverModel(self, label) :
        """
        Return value of solver for variable labelled label
        """
        node = self._getSchemeLabelNode(label)
        value = self.defaultValues()['solver']
        n = node.xmlGetNode('solver')
        if n:
            value = n['choice']
        return value


    @Variables.undoLocal
    def setSolverPrecision(self, label, value) :
        """
        Put value of solveur precision for variable labelled label
        """
        self.isPositiveFloat(value)
        node = self._getSchemeLabelNode(label)
        default = self.defaultValues()['solver_precision']
        if self._isAlpha(node):
            default = self.defaultValues()['solver_precision_alpha']
        elif self._isEnthalpy(node):
            default = self.defaultValues()['solver_precision_enthal']

        if value != default:
            node.xmlSetData('solver_precision', value)
        else:
            node.xmlRemoveChild('solver_precision')


    @Variables.noUndo
    def getSolverPrecision(self, label) :
        """
        Return value of solveur precision for variable labelled label
        """
        node = self._getSchemeLabelNode(label)

        default = self.defaultValues()['solver_precision']
        if self._isAlpha(node):
            default = self.defaultValues()['solver_precision_alpha']
        elif self._isEnthalpy(node):
            default = self.defaultValues()['solver_precision_enthal']

        value = node.xmlGetDouble('solver_precision')
        if value == None:
            value = default
        return value


    @Variables.undoLocal
    def setMaximumIteration(self, label, value) :
        """
        Put number of maximum iterations for variable labelled label
        """
        self.isInt(value)
        node = self._getSchemeLabelNode(label)
        default = self.defaultValues()['max_iter_number']
        if value != default :
            node.xmlSetData('max_iter_number', value)
        else:
            node.xmlRemoveChild('max_iter_number')


    @Variables.noUndo
    def getMaximumIteration(self, label) :
        """
        Return number of maximum iterations for variable labelled label
        """
        node = self._getSchemeLabelNode(label)
        value = node.xmlGetInt('max_iter_number')
        if value == None:
            value = self.defaultValues()['max_iter_number']
        return value


    def getPressureNodefields(self):
        for node in self.XMLNodeVariable.xmlGetNodeList('variable') :
            if node['label'] == 'Pressure':
                return node

#-------------------------------------------------------------------------------
# DefineUsersScalars test case
#-------------------------------------------------------------------------------

class NumericalParamEquatTestCase(ModelTest):
    """
    """
    def checkNumericalParamEquatInstantiation(self):
        """Check whether the NumericalParamEquationModel class could be instantiated"""
        model = None
        model = NumericalParamEquatModel(self.case)
        assert model != None, 'Could not instantiate NumericalParamEquationModel'


    def checkGetVariableList(self):
        """Check whether the NumericalParamEquationModel class could get the VariableList"""
        from code_saturne.model.MainFieldsModel import MainFieldsModel
        MainFieldsModel(self.case).addField()
        mdl = NumericalParamEquatModel(self.case)
        assert mdl.getVariableList() == ['enthalpy1', 'alpha1', 'U1', 'V1', 'W1'],\
            'Could not get VariableList'


    def checkGetandSetSchemeModel(self):
        """Check whether the NumericalParamEquationModel class could be set and get SchemeModel"""
        from code_saturne.model.MainFieldsModel import MainFieldsModel
        MainFieldsModel(self.case).addField()
        mdl = NumericalParamEquatModel(self.case)
        mdl.setSchemeModel('pressure','solu')
        doc = '''<variable field_id="none" label="Pressure" name="pressure">
                         <listing_printing status="on"/>
                         <postprocessing_recording status="on"/>
                         <scheme choice="solu"/>
                 </variable>'''
        assert mdl.getPressureNodefields() == self.xmlNodeFromString(doc),\
            'Could not set SchemeModel'
        assert mdl.getSchemeModel('Pressure') == 'solu',\
            'Could not get SchemeModel'


    def checkGetandSetSlopeTestStatus(self):
        """Check whether the NumericalParamEquationModel class could be set and get SlopeTestStatus"""
        from code_saturne.model.MainFieldsModel import MainFieldsModel
        MainFieldsModel(self.case).addField()
        mdl = NumericalParamEquatModel(self.case)
        mdl.setSlopeTestStatus('Pressure','off')
        doc = '''<variable field_id="none" label="Pressure" name="pressure">
                         <listing_printing status="on"/>
                         <postprocessing_recording status="on"/>
                         <slope_test status="off"/>
                 </variable>'''
        assert mdl.getPressureNodefields() == self.xmlNodeFromString(doc),\
            'Could not set SlopeTestStatus'
        assert mdl.getSlopeTestStatus('Pressure') == 'off',\
            'Could not get SlopeTestStatus'


    def checkGetandSetSolverModel(self):
        """Check whether the NumericalParamEquationModel class could be set and get SolverModel"""
        from code_saturne.model.MainFieldsModel import MainFieldsModel
        MainFieldsModel(self.case).addField()
        mdl = NumericalParamEquatModel(self.case)
        mdl.setSolverModel('Pressure','jacobi')
        doc = '''<variable field_id="none" label="Pressure" name="pressure">
                         <listing_printing status="on"/>
                         <postprocessing_recording status="on"/>
                         <solver choice="jacobi"/>
                 </variable>'''
        assert mdl.getPressureNodefields() == self.xmlNodeFromString(doc),\
            'Could not set SolverModel'
        assert mdl.getSolverModel('Pressure') == 'jacobi',\
            'Could not get SolverModel'


    def checkGetandSetSolverPrecision(self):
        """Check whether the NumericalParamEquationModel class could be set and get SolverPrecision"""
        from code_saturne.model.MainFieldsModel import MainFieldsModel
        MainFieldsModel(self.case).addField()
        mdl = NumericalParamEquatModel(self.case)
        mdl.setSolverPrecision('Pressure',56.23)
        doc = '''<variable field_id="none" label="Pressure" name="pressure">
                         <listing_printing status="on"/>
                         <postprocessing_recording status="on"/>
                         <solver_precision>
                                 56.23
                         </solver_precision>
                 </variable>'''
        assert mdl.getPressureNodefields() == self.xmlNodeFromString(doc),\
            'Could not set SolverPrecision'
        assert mdl.getSolverPrecision('Pressure') == 56.23,\
            'Could not get SolverPrecision'


    def checkGetandSetMaximumIteration(self):
        """Check whether the NumericalParamEquationModel class could be set and get MaximumIteration"""
        from code_saturne.model.MainFieldsModel import MainFieldsModel
        MainFieldsModel(self.case).addField()
        mdl = NumericalParamEquatModel(self.case)
        mdl.setMaximumIteration('Pressure',18)
        doc = '''<variable field_id="none" label="Pressure" name="pressure">
                         <listing_printing status="on"/>
                         <postprocessing_recording status="on"/>
                         <max_iter_number>
                                 18
                         </max_iter_number>
                 </variable>'''
        assert mdl.getPressureNodefields() == self.xmlNodeFromString(doc),\
            'Could not set MaximumIteration'
        assert mdl.getMaximumIteration('Pressure') == 18,\
            'Could not get MaximumIteration'


def suite():
    testSuite = unittest.makeSuite(NumericalParamEquatTestCase, "check")
    return testSuite


def runTest():
    print("NumericalParamEquatTestCase")
    runner = unittest.TextTestRunner()
    runner.run(suite())
